local addon, ns = ...
local cfg = CreateFrame("Frame")

--[[ START OF CONFIG ]]--
if IsAddOnLoaded("ElvUI") then
	-- ElvUI config --
	local E, C, L, DB = unpack(ElvUI)

	cfg.font = DB["media"].dmgfont
	cfg.fonts = E.Scale(30)
	cfg.fontf = "THINOUTLINE"
	cfg.pointy = 0
	cfg.pointx = -180
elseif IsAddOnLoaded("Tukui") then
	-- Tukui config --
	local T, C, L = unpack(Tukui)
	
	-- Special config for my edit
	if C["theme"] then
		cfg.font = C["theme"].uif
		cfg.fonts = T.Scale(C["theme"].uifs)
		cfg.fontf = C["theme"].uiff
		cfg.pointy = 0
		cfg.pointx = -196
	else
		cfg.font = C["media"].dmgfont
		cfg.fonts = T.Scale(30)
		cfg.fontf = "OUTLINE"
		cfg.pointy = 0
		cfg.pointx = -225
	end
else
	-- Normal config --
	cfg.font = "Interface\\AddOns\\CCP_Display\\font.ttf"	-- Font name
	cfg.fonts = 30											-- Font size
	cfg.fontf = "OUTLINE"									-- Font type ("OUTLINE", "THINOUTLINE", "THICKOUTLINE")
	cfg.pointy = 0											-- Horizontal position
	cfg.pointx = -150										-- Vertical position
end
--[[ END OF CONFIG ]]--

Combo = CreateFrame("Frame", "CCP")
local count = Combo:CreateFontString(nil, "OVERLAY")
count:SetFont(cfg.font,cfg.fonts,cfg.fontf)

local onEvent = function(self, event, ...)
	self[event](self, event, ...)
end

function Combo:Colorize(points) 
	if(points==1)then return .5,.7,1,1 end
	if(points==2)then return 0,1,0,1 end
	if(points==3)then return 1,1,0,1 end
	if(points==4)then return 1,.5,.2,1 end
	if(points==5)then return 1,0,0,1 end
	if(points==6)then return 1,0,1,1 end
	return 1,1,1,1
end

function Combo:UpdateComboPoints()
	if not UnitIsDead("target") then
		local points = GetComboPoints(UnitHasVehicleUI("player") and "vehicle" or "player", "target")
		count:SetText(points > 0 and points or "")
		count:SetVertexColor(Combo:Colorize(points))
	end
end

function Combo:PLAYER_ENTERING_WORLD(event)
	self:SetPoint("CENTER", UIParent, "CENTER", cfg.pointy, cfg.pointx)
	count:SetAllPoints(Combo)
	count:SetShadowColor(0,0,0,.8)
	count:SetShadowOffset(2.5,-2.5)	
end

function Combo:PLAYER_TARGET_CHANGED(event)
	self:UpdateComboPoints()
end

function Combo:UNIT_COMBO_POINTS(event, unit)
	if unit == "player" or unit == "vehicle" or unit == "target" then
		self:UpdateComboPoints()
	end
end

function Combo:UNIT_EXITED_VEHICLE(event, unit)
	if unit == "player" then
		self:UpdateComboPoints()
	end
end

Combo:SetWidth(cfg.fonts)
Combo:SetHeight(cfg.fonts)
Combo:SetScript("OnEvent", onEvent)
Combo:RegisterEvent("PLAYER_ENTERING_WORLD")
Combo:RegisterEvent("PLAYER_TARGET_CHANGED")
Combo:RegisterEvent("UNIT_COMBO_POINTS")
Combo:RegisterEvent("UNIT_EXITED_VEHICLE")

-- Hide Blizzards's Combo Frame
ComboFrame:UnregisterEvent("PLAYER_TARGET_CHANGED")
ComboFrame:UnregisterEvent("UNIT_COMBO_POINTS")

ns.cfg = cfg -- Don't change this !!