local T, C, L = unpack(select(2, ...)) -- Import: T - functions, constants, variables; C - config; L - locales
if not C["datatext"].location then return end

--------------------------------------------------------------------
 -- Location
--------------------------------------------------------------------

local size = T.buttonsize - 10

local font, fonts, fontf 

if C["theme"].pfont then
	font, fonts, fontf = C["media"].pixelfont, C["theme"].pfonts, C["theme"].pfontf
else
	font, fonts, fontf = C["theme"].dataf, C["theme"].datafs, C["theme"].dataff
end

local location = CreateFrame("Frame", "TukuiLocation", UIParent)
local xcoords = CreateFrame("Frame", "TukuiXCoords", UIParent)
local ycoords = CreateFrame("Frame", "TukuiYCoords", UIParent)

if C["others"].viewport then
	location:SetParent(TukuiTopBar)
	location:SetHeight(TukuiTopBar:GetHeight())
	location:SetPoint('CENTER', TukuiTopBar, 0, 0)
	location:SetPoint('TOP', TukuiTopBar, 0, 0)
	location:SetPoint('BOTTOM', TukuiTopBar)
	
	xcoords:SetParent(location)
	xcoords:SetHeight(location:GetHeight())
	xcoords:SetPoint('LEFT', location, -30, 0)
	xcoords:SetPoint('TOP', location, 0, 0)
	xcoords:SetPoint('BOTTOM', location)
	
	ycoords:SetParent(location)
	ycoords:SetHeight(location:GetHeight())
	ycoords:SetPoint('RIGHT', location, 30, 0)
	ycoords:SetPoint('TOP', location, 0, 0)
	ycoords:SetPoint('BOTTOM', location)
else
	location:CreatePanel("Default", true, size, size, "TOP", UIParent, "TOP", 0, -10)
	xcoords:CreatePanel("Default", true, size, size, "RIGHT", location, "LEFT", -T.buttonspacing, 0)
	ycoords:CreatePanel("Default", true, size, size, "LEFT", location, "RIGHT", T.buttonspacing, 0)
	
	location:SetFrameLevel(2)
	xcoords:SetFrameLevel(2)
	ycoords:SetFrameLevel(2)
end

local lText = location:CreateFontString(nil, "LOW")
lText:SetFont(font, fonts, fontf)
lText:SetShadowOffset(1, -1)
lText:SetPoint("CENTER", location, 0, 0)
 
local function OnEvent(self, event)
	local loc = GetMinimapZoneText()
	local pvpType, isFFA, zonePVPStatus = GetZonePVPInfo()

	if (pvpType == "sanctuary") then
		loc = "|cff69C9EF"..loc.."|r" -- light blue
	elseif (pvpType == "friendly") then
		loc = "|cff00ff00"..loc.."|r" -- green
	elseif (pvpType == "contested") then
		loc = "|cffffff00"..loc.."|r" -- yellow
	elseif (pvpType == "hostile" or pvpType == "combat" or pvpType == "arena" or not pvpType) then
		loc = "|cffff0000"..loc.."|r" -- red
	else
		loc = loc -- white
	end

	lText:SetText(loc)
	location:SetWidth(lText:GetWidth() + (size))
end

location:RegisterEvent("PLAYER_ENTERING_WORLD")
location:RegisterEvent("ZONE_CHANGED_NEW_AREA")
location:RegisterEvent("ZONE_CHANGED")
location:RegisterEvent("ZONE_CHANGED_INDOORS")
location:HookScript("OnEvent", OnEvent)

local xcoordstext = xcoords:CreateFontString(nil, "LOW")
xcoordstext:SetFont(font, fonts, fontf)
xcoordstext:SetShadowOffset(1, -1)
xcoordstext:Point("CENTER", xcoords, 0, 0)
	
local ycoordstext  = ycoords:CreateFontString(nil, "LOW")
ycoordstext:SetFont(font, fonts, fontf)
ycoordstext:SetShadowOffset(1, -1)
ycoordstext:Point("CENTER", ycoords, T.mult, 0)

--[[if C["others"].viewport then
	location:SetBackdropColor(0, 0, 0, 0)
	location:SetBackdropBorderColor(0, 0, 0, 0)
	xcoords:SetBackdropColor(0, 0, 0, 0)
	xcoords:SetBackdropBorderColor(0, 0, 0, 0)
	ycoords:SetBackdropColor(0, 0, 0, 0)
	ycoords:SetBackdropBorderColor(0, 0, 0, 0)
end]]

local ela,go = 0,false

local Update = function(self,t)
	ela = ela - t
	if ela > 0 then return end
	local x,y = GetPlayerMapPosition("player")
	local xt,yt
	x = math.floor(100 * x)
	y = math.floor(100 * y)
	if x == 0 and y == 0 then
		xcoordstext:SetText("")
		ycoordstext:SetText("")
			
		xcoords:SetAlpha(0)
		ycoords:SetAlpha(0)
	else
		if x < 10 then
			xt = "0"..x
		else
			xt = x
		end
		if y < 10 then
			yt = "0"..y
		else
			yt = y
		end
		xcoordstext:SetText(xt)
		ycoordstext:SetText(yt)
			
		xcoords:SetAlpha(1)
		ycoords:SetAlpha(1)
		xcoords:SetWidth(xcoordstext:GetWidth() + 20)
		ycoords:SetWidth(ycoordstext:GetWidth() + 20)
	end
	ela = .2
end
xcoords:HookScript("OnUpdate", Update)