local T, C, L = unpack(select(2, ...)) -- Import: T - functions, constants, variables; C - config; L - locales
-- Editless skin by MrRuben5, edited by Dajova
if not (IsAddOnLoaded("NugRunning")) then return end

local bettertime = true
local modifyfont = true
local fontflags = {C.theme.uff, C.theme.uffs, C.theme.ufff}
local color = "|cffff0000"
------------------------------------------------------------------------------
local unpack = unpack
local format = format
local _

-- helper function for better time text
local FormatTime
do
	local day, hour, minute = 86400, 3600, 60
	FormatTime = function(s)
		if s >= day then
			return format("%d%sd|r", ceil(s / day), color)
		elseif s >= hour then
			return format("%d%sh|r", ceil(s / hour), color)
		elseif s >= minute then
			return format("%d%sm|r", ceil(s / minute), color)
		elseif s >= minute / 12 then
			return floor(s)
		end
		return format("%.1f", s)
	end
end

-- better time text
-- not called when "bettertime" is false
local SetTimeText = function(self, _, timeDigits)
	self:SetText(FormatTime(timeDigits))
end

-- Overrule font when NugRunning updates settings
-- not called when "modifyfont" is false
local OrigTimerOnSettingsChanged
local TimerOnSettingsChanged = function(self)
    OrigTimerOnSettingsChanged(self)
    self.timeText:SetFont(unpack(fontflags))
    self.spellText:SetFont(unpack(fontflags))
    self.stacktext:SetFont(unpack(fontflags))
end

-- Only set alpha on the bar, the icon and stacks when target is changed
local ModifyElementAlpha
do
	local _
	local elements = {"icon", "stacktext", "bar"}
	ModifyElementAlpha = function(self, alpha)
		for _, elm in pairs(elements) do
			self[elm]:SetAlpha(alpha)
		end
	end
end

-- Hook bar creation function
hooksecurefunc(NugRunning, 'BarFrame', function(f)
	f:SetTemplate("Transparent")
	f:CreateShadow("Default")
	
	local ic = f.icon:GetParent()
	
	f.icon:ClearAllPoints()
	f.icon:Point("BOTTOMLEFT", ic, 2, 2)
	f.icon:Point("TOPRIGHT", ic, -2, -2)
	
	f.bar:ClearAllPoints()
	f.bar:Point('TOPRIGHT', -2, -2)
	f.bar:Point('BOTTOMRIGHT', -2, 2)
	
	f.bar:SetStatusBarTexture(C.media.normTex)
	f.bar.bg:SetTexture(C.media.normTex)
	
	f.SetAlpha = ModifyElementAlpha
	
	-- Better time formatting
	if bettertime then
		f.timeText.SetFormattedText = SetTimeText
	end
	
	-- Checking for variables...
	local db = NRunDB
	if db then
		db.width = 200
		db.height = 20
		if C["unitframes"].cbinside then
			db.anchor.y = 200
			db.anchor.x = -100
			db.anchor.to = "BOTTOM"
			db.anchor.point = "BOTTOM"
		else
			db.anchor.y = -235
			db.anchor.x = -100
			db.anchor.to = "CENTER"
			db.anchor.point = "CENTER"
		end
	end	
	
	-- font strings...
	if modifyfont then
		OrigTimerOnSettingsChanged = OrigTimerOnSettingsChanged or f.OnSettingsChanged
		f.OnSettingsChanged = TimerOnSettingsChanged
		f:OnSettingsChanged()
	end
end)