local ADDON_NAME, ns = ...
local oUF = ns.oUF or oUF
assert(oUF, "Tukui was unable to locate oUF install.")

ns._Objects = {}
ns._Headers = {}

local T, C, L = unpack(select(2, ...)) -- Import: T - functions, constants, variables; C - config; L - locales
if not C["unitframes"].enable == true then return end

------------------------------------------------------------------------
--	local variables
------------------------------------------------------------------------

local font, fonts, fontf

if C["theme"].pfont then
	font, fonts, fontf = C["media"].pixelfont, C["theme"].pfonts, C["theme"].pfontf
else
	font, fonts, fontf = C["theme"].uff, C["theme"].uffs, C["theme"].ufff
end

local normTex = C["media"].normTex

local width
if TukuiPetBar then
	width = TukuiPetBar:GetWidth() - 97
else
	width = T.DataWidth / 1.5
end
local height = T.buttonsize - 10
local asize = T.buttonsize - 4.1

------------------------------------------------------------------------
--	Layout
------------------------------------------------------------------------

local function Shared(self, unit)
	-- set our own colors
	self.colors = T.oUF_colors
	
	-- register click
	self:RegisterForClicks("AnyUp")
	self:SetScript('OnEnter', UnitFrame_OnEnter)
	self:SetScript('OnLeave', UnitFrame_OnLeave)
		
	-- menu? lol
	self.menu = T.SpawnMenu

	------------------------------------------------------------------------
	--	Features we want for all units at the same time
	------------------------------------------------------------------------
	
	-- here we create an invisible frame for all element we want to show over health/power.
	local InvFrame = CreateFrame("Frame", nil, self)
	InvFrame:SetFrameStrata("HIGH")
	InvFrame:SetFrameLevel(6)
	InvFrame:SetAllPoints()
	
	-- symbols, now put the symbol on the frame we created above.
	local RaidIcon = InvFrame:CreateTexture(nil, "OVERLAY")
	RaidIcon:SetTexture("Interface\\AddOns\\Tukui\\medias\\textures\\raidicons.blp") -- thx hankthetank for texture
	RaidIcon:SetHeight(asize)
	RaidIcon:SetWidth(asize)
	RaidIcon:SetPoint("TOP", 0, 8)
	self.RaidIcon = RaidIcon
	
	------------------------------------------------------------------------
	--	Player and Target units layout (mostly mirror'd)
	------------------------------------------------------------------------
	
	if (unit == "player" or unit == "target") then	
		-- health
		local health = CreateFrame('StatusBar', nil, self)
		health:SetFrameLevel(5)
		health:SetHeight(height * 1.5)
		health:SetPoint("TOPLEFT", T.mult, -T.mult)
		health:SetPoint("TOPRIGHT", -T.mult, T.mult)
		health:SetStatusBarTexture(normTex)
		health:GetStatusBarTexture():SetHorizTile(false)
		self.Health = health
		
		local healthBg = health:CreateTexture(nil, "BORDER")
		healthBg:SetAllPoints()
		healthBg:SetTexture(unpack(C["media"].bgcolor))
		self.Health.bg = healthBg
		
		local healthB = CreateFrame("Frame", nil, health)
		healthB:SetFrameLevel(health:GetFrameLevel() - 1)
		healthB:Point("TOPLEFT", -2, 2)
		healthB:Point("BOTTOMRIGHT", 2, -2)
		healthB:SetTemplate("Default", true)
		healthB:CreateShadow("Default")
		self.Health.border = healthB
						
		-- power
		local power = CreateFrame('StatusBar', nil, self)
		power:SetHeight(height/2)
		power:SetPoint("TOPLEFT", healthB, "BOTTOMLEFT", T.Scale(2), -T.buttonspacing*2)
		power:SetPoint("TOPRIGHT", healthB, "BOTTOMRIGHT", T.Scale(-2), -T.buttonspacing*2)
		power:SetStatusBarTexture(normTex)
		power:GetStatusBarTexture():SetHorizTile(false)
		self.Power = power
		
		local powerBg = power:CreateTexture(nil, "BORDER")
		powerBg:SetAllPoints()
		powerBg:SetTexture(unpack(C["media"].bgcolor))
		powerBg.multiplier = 0.3
		self.Power.bg = powerBg
		
		local powerB = CreateFrame("Frame", nil, power)
		powerB:SetFrameLevel(power:GetFrameLevel() - 1)
		powerB:Point("TOPLEFT", -2, 2)
		powerB:Point("BOTTOMRIGHT", 2, -2)
		powerB:SetTemplate("Default", true)
		powerB:CreateShadow("Default")
		self.Power.border = powerB	
				
		-- create a panel
		local panel = CreateFrame("Frame", nil, self)
		panel:CreatePanel("Default", true, 1, height, "BOTTOM", self, "BOTTOM", 0, 0)
		panel:SetFrameLevel(self.Health:GetFrameLevel())
		panel:ClearAllPoints()
		panel:SetPoint("BOTTOMLEFT", healthB, "TOPLEFT", 0, T.buttonspacing)
		panel:SetPoint("BOTTOMRIGHT", healthB, "TOPRIGHT", 0, T.buttonspacing)
		self.panel = panel

		-- portraits
		if (C["unitframes"].charportrait == true) then
			local portrait = CreateFrame("PlayerModel", self:GetName().."_Portrait", self)
		    portrait.parent = panel
			portrait:SetFrameLevel(panel:GetFrameLevel() + 1)
		    portrait:SetPoint("TOPLEFT", panel, "TOPLEFT", 2, -2)
		    portrait:SetPoint("BOTTOMRIGHT", panel, "BOTTOMRIGHT", -2, 2)
			table.insert(self.__elements, T.HidePortrait)
      
				portrait.PostUpdate = function(self) -- hide the portrait partionally for a cool effect
				self:SetAlpha(0) 
				self:SetAlpha(0.25)       
			end
			self.Portrait = portrait
		end
		
		health.frequentUpdates = true
		power.frequentUpdates = true
		
		if C["unitframes"].showsmooth == true then
			health.Smooth = true
			power.Smooth = true
		end
		
		if C["unitframes"].unicolor == false then
			health.colorTapping = true
			health.colorDisconnected = true
			health.colorReaction = true
			health.colorClass = true
			healthBg.multiplier = 0.3
			
			power.colorPower = true
		else
			health.colorTapping = false
			health.colorDisconnected = false
			health.colorClass = false
			health.colorReaction = false
			health:SetStatusBarColor(unpack(C["media"].barcolor))
			health.bg:SetVertexColor(unpack(C["media"].bgcolor))
			
			power.multiplier = 0.5
			power.colorTapping = true
			power.colorDisconnected = true
			power.colorClass = true
			power.colorReaction = true
		end
		
		health.value = T.SetFontString(health, font, fonts, fontf)
		health.value:Point("RIGHT", panel, "RIGHT", -4, 0)
    
		health.percent = T.SetFontString(health, font, health:GetHeight(), fontf)
		if unit == "player" then
			health.percent:Point("LEFT", health, "RIGHT", 6, 0)
			health.percent:SetShadowOffset(2, -2)
		elseif unit == "target" then
			health.percent:Point("RIGHT", health, "LEFT", -6, 0)
			health.percent:SetShadowOffset(-2, -2)
		end
		health.PostUpdate = T.PostUpdateHealth
		
		power.value = T.SetFontString(power, font, fonts, fontf)
		power.value:Point("LEFT", self.panel, "LEFT", 4, 0)

		power.percent = T.SetFontString(panel, font, panel:GetHeight(), fontf)
		if unit == "player" then
			power.percent:Point("LEFT", panel, "RIGHT", 6, 0)
			power.percent:SetShadowOffset(1, -1)
		elseif unit == "target" then
			power.percent:Point("RIGHT", panel, "LEFT", -6, 0)
			power.percent:SetShadowOffset(-1, -1)
		end
		power.PreUpdate = T.PreUpdatePower
		power.PostUpdate = T.PostUpdatePower
		
		if T.myclass == "PRIEST" and C["unitframes"].weakenedsoulbar then
			local ws = CreateFrame("StatusBar", self:GetName().."_WeakenedSoul", power)
			ws:SetAllPoints(power)
			ws:SetStatusBarTexture(normTex)
			ws:GetStatusBarTexture():SetHorizTile(false)
			ws:SetBackdropColor(0, 0, 0, 0)
			ws:SetStatusBarColor(191/255, 10/255, 10/255)
			self.WeakenedSoul = ws
		end
					
		if (unit == "player") then
			-- combat icon
			local Combat = health:CreateTexture(nil, "OVERLAY")
			Combat:Height(asize/1.25)
			Combat:Width(asize/1.25)
			Combat:SetPoint("TOPRIGHT", 2, 8)
			Combat:SetVertexColor(0.69, 0.31, 0.31)
			self.Combat = Combat
			
			-- resting icon
			local Resting = panel:CreateTexture(nil, "OVERLAY")
			Resting:SetHeight(asize/1.5)
			Resting:SetWidth(asize/1.5)
			Resting:SetPoint("CENTER", 0, 2)
			Resting:SetTexture([=[Interface\CharacterFrame\UI-StateIcon]=])
			Resting:SetTexCoord(0, 0.5, 0, 0.421875)
			self.Resting = Resting

			-- custom info (low mana warning)
			FlashInfo = CreateFrame("Frame", "TukuiFlashInfo", self)
			FlashInfo:SetScript("OnUpdate", T.UpdateManaLevel)
			FlashInfo.parent = self
			FlashInfo:SetAllPoints(panel)
			FlashInfo.ManaLevel = T.SetFontString(FlashInfo, font, fonts, fontf)
			FlashInfo.ManaLevel:SetPoint("CENTER", panel, "CENTER", 0, 0)
			self.FlashInfo = FlashInfo
			
			-- pvp status text
			local status = T.SetFontString(panel, font, fonts, fontf)
			status:SetPoint("CENTER", panel, "CENTER", 0, 0)
			status:SetTextColor(0.69, 0.31, 0.31, 0)
			self.Status = status
			self:Tag(status, "[pvp]")
			
			-- leader icon
			local Leader = InvFrame:CreateTexture(nil, "OVERLAY")
			Leader:Height(asize/2)
			Leader:Width(asize/2)
			Leader:Point("TOPLEFT", 2, 8)
			self.Leader = Leader
			
			-- master looter
			local MasterLooter = InvFrame:CreateTexture(nil, "OVERLAY")
			MasterLooter:Height(asize/2)
			MasterLooter:Width(asize/2)
			self.MasterLooter = MasterLooter
			self:RegisterEvent("PARTY_LEADER_CHANGED", T.MLAnchorUpdate)
			self:RegisterEvent("PARTY_MEMBERS_CHANGED", T.MLAnchorUpdate)
						
			-- show druid mana when shapeshifted in bear, cat or whatever
			if T.myclass == "DRUID" then
				CreateFrame("Frame"):SetScript("OnUpdate", function() T.UpdateDruidMana(self) end)
				local DruidMana = T.SetFontString(panel, font, panel:GetHeight(), fontf)
				DruidMana:SetTextColor(1, 0.49, 0.04)
				self.DruidMana = DruidMana
			end
			
			if C["unitframes"].classbar then
				if T.myclass == "DRUID" then			
					local eclipseBar = CreateFrame('Frame', nil, self)
					eclipseBar:Point("BOTTOMLEFT", panel, "TOPLEFT", 2, 5)
					if T.lowversion then
						eclipseBar:SetSize(width / 2, 10)
					else
						eclipseBar:SetSize(width - (T.Scale(2)), 10)
					end
					eclipseBar:SetFrameStrata("MEDIUM")
					eclipseBar:SetFrameLevel(8)
					eclipseBar:SetScript("OnShow", function() T.EclipseDisplay(self, false) end)
					eclipseBar:SetScript("OnUpdate", function() T.EclipseDisplay(self, true) end) -- just forcing 1 update on login for buffs/shadow/etc.
					eclipseBar:SetScript("OnHide", function() T.EclipseDisplay(self, false) end)
					
					local eclipseBg = CreateFrame("Frame", nil, eclipseBar)
					eclipseBg:Point("TOPLEFT", eclipseBar, "TOPLEFT", -2, 2)
					eclipseBg:Point("BOTTOMRIGHT", eclipseBar, "BOTTOMRIGHT", 2, -2)
					eclipseBg:SetFrameLevel(eclipseBar:GetFrameLevel() - 1)
					
					local lunarBar = CreateFrame('StatusBar', nil, eclipseBar)
					lunarBar:SetPoint('LEFT', eclipseBar, 'LEFT', 0, 0)
					lunarBar:SetSize(eclipseBar:GetWidth(), eclipseBar:GetHeight())
					lunarBar:SetStatusBarTexture(normTex)
					lunarBar:SetStatusBarColor(.30, .52, .90)
					eclipseBar.LunarBar = lunarBar

					local solarBar = CreateFrame('StatusBar', nil, eclipseBar)
					solarBar:SetPoint('LEFT', lunarBar:GetStatusBarTexture(), 'RIGHT', 0, 0)
					solarBar:SetSize(eclipseBar:GetWidth(), eclipseBar:GetHeight())
					solarBar:SetStatusBarTexture(normTex)
					solarBar:SetStatusBarColor(.80, .82,  .60)
					eclipseBar.SolarBar = solarBar

					local eclipseBarText = eclipseBar:CreateFontString(nil, 'OVERLAY')
					eclipseBarText:Point('TOP', self)
					eclipseBarText:Point('BOTTOM', self)
					eclipseBarText:SetFont(font, fonts, fontf)
					eclipseBar.PostUpdatePower = T.EclipseDirection
					
					-- hide "low mana" text on load if eclipseBar is show
					if eclipseBar and eclipseBar:IsShown() then FlashInfo.ManaLevel:SetAlpha(0) end
					
					eclipseBg:SetTemplate("Default", true)
					eclipseBg:CreateShadow("Default")
					
					self.EclipseBar = eclipseBar
					self.EclipseBar.Text = eclipseBarText
				end

				-- holy power bar or shard bar
				if ((T.myclass == "WARLOCK" and C["unitframes"].soulshardbar) or (T.myclass == "PALADIN" and C["unitframes"].holypowerbar)) then					
					local bars = CreateFrame("Frame", nil, self)	
					local bF = CreateFrame("Frame", nil, self)
				
					for i = 1, 3 do		
						bF[i] = CreateFrame("Frame", nil, self)
						bF[i]:SetFrameLevel(health:GetFrameLevel() + 1)
						bF[i]:Height(10)
						bF[i]:SetWidth((width + 4) / 3)
					
						if (i == 1) then
							bF[i]:SetPoint("BOTTOMLEFT", self.panel, "TOPLEFT", 0, T.buttonspacing)
						else
							bF[i]:SetPoint("TOPLEFT", bF[i-1], "TOPRIGHT", -T.mult, 0)
						end

						bars[i] = CreateFrame("StatusBar", self:GetName().."_Shard"..i, self)
						bars[i]:Point("TOPLEFT", bF[i], "TOPLEFT", 2, -2)
						bars[i]:Point("BOTTOMRIGHT", bF[i], "BOTTOMRIGHT", -2, 2)
						bars[i]:SetFrameLevel(health:GetFrameLevel() + 2)
						bars[i]:SetStatusBarTexture(normTex)			

						bars[i].bg = bars[i]:CreateTexture(nil, 'BORDER')			
						bars[i].bg:SetTexture(bars[i]:GetStatusBarTexture())					
						bars[i].bg.multiplier = 0.3
						
						bF[i]:SetTemplate("Default", true)
						bF[i]:CreateShadow("Default")

						if T.myclass == "WARLOCK" then
							bars[i]:SetStatusBarColor(1, 0, 1, 1)
						elseif T.myclass == "PALADIN" then
							bars[i]:SetStatusBarColor(1, 1, 0, 1)
						end
					end
				
					if T.myclass == "WARLOCK" then
						bars.Override = T.UpdateShards				
						self.SoulShards = bars
					elseif T.myclass == "PALADIN" then
						bars.Override = T.UpdateHoly
						self.HolyPower = bars
					end
				end

				-- deathknight runes
				if T.myclass == "DEATHKNIGHT" then				
					local Runes = CreateFrame("Frame", nil, self)
					local rB = CreateFrame("Frame", nil, self)
					for i = 1, 6 do
						rB[i] = CreateFrame("Frame", nil, self)
						rB[i]:Height(10)
						rB[i]:SetWidth((width + 7) / 6)
						rB[i]:SetFrameLevel(health:GetFrameLevel() + 1)
						
						if (i == 1) then
							rB[i]:SetPoint("BOTTOMLEFT", self.panel, "TOPLEFT", 0, T.buttonspacing)
						else
							rB[i]:SetPoint("TOPLEFT", rB[i-1], "TOPRIGHT", -T.mult, 0)
						end

						Runes[i] = CreateFrame("StatusBar", self:GetName().."_Runes"..i, self)
						Runes[i]:Point("TOPLEFT", rB[i], "TOPLEFT", 2, -2)
						Runes[i]:Point("BOTTOMRIGHT", rB[i], "BOTTOMRIGHT", -2, 2)
						Runes[i]:SetFrameLevel(health:GetFrameLevel() + 2)
						Runes[i]:SetStatusBarTexture(normTex)			
						Runes[i]:GetStatusBarTexture():SetHorizTile(false)
												
						rB[i]:SetTemplate("Default", true)
						rB[i]:CreateShadow("Default")
					end
					
					self.Runes = Runes
				end
				
				-- shaman totem bar
				if T.myclass == "SHAMAN" and not IsAddOnLoaded("TotemTimers") then					
					local TotemBar = {}
					local tB = CreateFrame("Frame", nil, self)
					TotemBar.Destroy = true
					for i = 1, 4 do
						tB[i] = CreateFrame("Frame", nil, self)
						tB[i]:SetFrameLevel(health:GetFrameLevel() + 1)
						tB[i]:Height(10)
						tB[i]:SetWidth((width + 5) / 4)
					
						if (i == 1) then
							tB[i]:Point("BOTTOMLEFT", self.panel, "TOPLEFT", 0, 3)
						else
							tB[i]:Point("TOPLEFT", tB[i-1], "TOPRIGHT", -1, 0)
						end

						TotemBar[i] = CreateFrame("StatusBar", self:GetName().."_Totem"..i, self)
						TotemBar[i]:Point("TOPLEFT", tB[i], "TOPLEFT", 2, -2)
						TotemBar[i]:Point("BOTTOMRIGHT", tB[i], "BOTTOMRIGHT", -2, 2)
						TotemBar[i]:SetFrameLevel(health:GetFrameLevel() + 2)
						TotemBar[i]:SetStatusBarTexture(normTex)			
						TotemBar[i]:GetStatusBarTexture():SetHorizTile(false)
						TotemBar[i]:SetMinMaxValues(0, 1)
						
						TotemBar[i].bg = TotemBar[i]:CreateTexture(nil, 'BORDER')			
						TotemBar[i].bg:SetTexture(TotemBar[i]:GetStatusBarTexture())
						TotemBar[i].bg:Point("TOPLEFT", TotemBar[i], "TOPLEFT", 2, -2)
						TotemBar[i].bg:Point("BOTTOMRIGHT", TotemBar[i], "BOTTOMRIGHT", -2, 2)
						
						tB[i]:SetTemplate("Default", true)
						tB[i]:CreateShadow("Default")
					end
					self.TotemBar = TotemBar
				end
			end
			
			-- script for pvp status and low mana
			self:SetScript("OnEnter", function(self)
				if self.EclipseBar and self.EclipseBar:IsShown() then 
					self.EclipseBar.Text:Hide()
				end
				FlashInfo.ManaLevel:SetAlpha(0) 
				status:SetAlpha(1) 
				UnitFrame_OnEnter(self) 
			end)
			self:SetScript("OnLeave", function(self) 
				if self.EclipseBar and self.EclipseBar:IsShown() then 
					self.EclipseBar.Text:Show()
				end
				FlashInfo.ManaLevel:SetAlpha(1)
				status:SetAlpha(0) 
				UnitFrame_OnLeave(self) 
			end)
		end
		
		if (unit == "target") then			
			-- Unit name on target
			local Name = health:CreateFontString(nil, "OVERLAY")
			Name:Point("LEFT", panel, "LEFT", 4, 0)
			Name:SetJustifyH("LEFT")
			Name:SetFont(font, fonts, fontf)
			Name:SetShadowOffset(1, -1)
			self.Name = Name
			
			if C["unitframes"].unicolor == false then
				self:Tag(Name, '[Tukui:namelong][Tukui:diffcolor] [level][shortclassification]')
			else
				self:Tag(Name, '[Tukui:getnamecolor][Tukui:namelong][Tukui:diffcolor] [level][shortclassification]')
			end
			
			-- combo point bars on target
			if (T.myclass == "ROGUE" or T.myclass == "DRUID") then
				local CPoints = {}
				CPoints.unit = PlayerFrame.unit
				local cB = CreateFrame("Frame", nil, self)
				local bars = MAX_COMBO_POINTS
				for i = 1, bars do
					cB[i] = CreateFrame("Frame", nil, self)
					cB[i]:Height(10)
					cB[i]:SetWidth((width + 6) / bars)
					cB[i]:SetFrameLevel(health:GetFrameLevel() + 1)
					
					if (i == 1) then
						cB[i]:SetPoint("BOTTOMLEFT", self.panel, "TOPLEFT", 0, (asize * 2) + (T.buttonspacing * 3))
					else
						cB[i]:SetPoint("TOPLEFT", cB[i-1], "TOPRIGHT", -T.mult, 0)
					end
					
					CPoints[i] = CreateFrame("StatusBar", self:GetName().."_Combo"..i, self)
					CPoints[i]:Point("TOPLEFT", cB[i], "TOPLEFT", 2, -2)
					CPoints[i]:Point("BOTTOMRIGHT", cB[i], "BOTTOMRIGHT", -2, 2)
					CPoints[i]:SetFrameLevel(health:GetFrameLevel() + 2)
					CPoints[i]:SetStatusBarTexture(normTex)
					
					CPoints[i].bg = CPoints[i]:CreateTexture(nil, 'BORDER')
					CPoints[i].bg:SetTexture(CPoints[i]:GetStatusBarTexture())
					CPoints[i].bg:SetAlpha(.15)
					
					if i == 1 then
						CPoints[i]:SetStatusBarColor(0.69, 0.31, 0.31)
					elseif i == 2 then
						CPoints[i]:SetStatusBarColor(0.69, 0.31, 0.31)
					elseif i == 3 then
						CPoints[i]:SetStatusBarColor(0.65, 0.63, 0.35)
					elseif i == 4 then
						CPoints[i]:SetStatusBarColor(0.65, 0.63, 0.35)
					elseif i == 5 then
						CPoints[i]:SetStatusBarColor(0.33, 0.59, 0.33)
					end
					
					cB[i]:SetTemplate("Default", true)
					cB[i]:CreateShadow("Default")
					
					-- some extra code to hide it ooc and if target has 0 points, taken from sCombo
					cB[i]:RegisterEvent("PLAYER_ENTERING_WORLD")
					cB[i]:RegisterEvent("UNIT_COMBO_POINTS")
					cB[i]:RegisterEvent("PLAYER_TARGET_CHANGED")
					cB[i]:SetScript("OnEvent", function(self, event)
					local points, pt = 0, GetComboPoints("player", "target")
						if pt == points then
							cB[i]:Hide()
						elseif pt > points then
							for i = points + 1, pt do
								cB[i]:Show()
							end
						else
							for i = pt + 1, points do
								cB[i]:Hide()
							end
						end
						points = pt	
					end)
				end
				
				self.CPoints = CPoints
			end
		end
			
		if (unit == "target" and C["unitframes"].targetauras) or (unit == "player" and C["unitframes"].playerauras) then
			local buffs = CreateFrame("Frame", nil, self.panel)
			local debuffs = CreateFrame("Frame", nil, self.panel)
			
			buffs:SetPoint("BOTTOMLEFT", self.panel, "TOPLEFT", 0, T.buttonspacing)
			
			if T.lowversion then
				buffs:SetHeight(asize*0.5)
				buffs:SetWidth(width/2)
				buffs.size = asize*0.5
				buffs.num = 8
				
				debuffs:SetHeight(asize*0.5)
				debuffs:SetWidth(width/2)
				debuffs:SetPoint("BOTTOMLEFT", buffs, "TOPLEFT", T.mult, T.buttonspacing)
				debuffs.size = asize*0.5
				debuffs.num = 8
			else				
				buffs:SetHeight(asize)
				buffs:SetWidth(width)
				buffs.size = asize
				buffs.num = 8
				
				debuffs:SetHeight(asize)
				debuffs:SetWidth(width)
				debuffs:SetPoint("BOTTOMLEFT", buffs, "TOPLEFT", T.mult, T.buttonspacing)
				debuffs.size = asize
				debuffs.num = 8
			end
						
			buffs.spacing = T.buttonspacing
			buffs.initialAnchor = 'TOPLEFT'
			buffs.PostCreateIcon = T.PostCreateAura
			buffs.PostUpdateIcon = T.PostUpdateAura
			self.Buffs = buffs	
						
			debuffs.spacing = T.buttonspacing
			debuffs.initialAnchor = 'TOPRIGHT'
			debuffs["growth-y"] = "UP"
			debuffs["growth-x"] = "LEFT"
			debuffs.PostCreateIcon = T.PostCreateAura
			debuffs.PostUpdateIcon = T.PostUpdateAura
			
			-- an option to show only our debuffs on target
			if unit == "target" then
				debuffs.onlyShowPlayer = C.unitframes.onlyselfdebuffs
			end
			
			self.Debuffs = debuffs
		end
		
		-- cast bar for player and target
		if (C["unitframes"].unitcastbar == true) then
			-- castbar of player and target
			local castbar = CreateFrame("StatusBar", self:GetName().."_CastBar", self)
			castbar:SetStatusBarTexture(normTex)
			castbar:GetStatusBarTexture():SetHorizTile(false)
			
			if C["unitframes"].cbinside then
				castbar.bg = castbar:CreateTexture(nil, "BORDER")
				castbar.bg:SetAllPoints(castbar)
				castbar.bg:SetTexture(normTex)
				castbar.bg:SetVertexColor(unpack(C["media"].backdropcolor))
				castbar:SetFrameLevel(panel:GetFrameLevel() + 2)
				castbar:Point("TOPLEFT", panel, 2, -2)
				castbar:Point("BOTTOMRIGHT", panel, -2, 2)
			
				castbar.CustomTimeText = T.CustomCastTimeText
				castbar.CustomDelayText = T.CustomCastDelayText
				castbar.PostCastStart = T.CheckCast
				castbar.PostChannelStart = T.CheckChannel

				castbar.time = T.SetFontString(castbar, font, fonts, fontf)
				castbar.time:Point("RIGHT", panel, "RIGHT", -4, 0)
				castbar.time:SetTextColor(0.84, 0.75, 0.65)
				castbar.time:SetJustifyH("RIGHT")

				castbar.Text = T.SetFontString(castbar, font, fonts, fontf)
				castbar.Text:Point("LEFT", panel, "LEFT", 4, 0)
				castbar.Text:SetWidth(width - (T.buttonsize * 2))
				castbar.Text:SetTextColor(0.84, 0.75, 0.65)
			
				if C["unitframes"].cbicons and not T.lowversion then
					castbar.button = CreateFrame("Frame", nil, castbar)
					castbar.button:SetSize(height * 1.74, height * 1.74)
					castbar.button:SetTemplate("Default", true)
					castbar.button:CreateShadow("Default")

					castbar.icon = castbar.button:CreateTexture(nil, "ARTWORK")
					castbar.icon:Point("TOPLEFT", castbar.button, 2, -2)
					castbar.icon:Point("BOTTOMRIGHT", castbar.button, -2, 2)
					castbar.icon:SetTexCoord(0.08, 0.92, 0.08, .92)
			
					if unit == "player" then
						castbar.button:SetPoint("TOPRIGHT", castbar, "TOPLEFT", -T.buttonspacing*2, 1)
					elseif unit == "target" then
						castbar.button:SetPoint("TOPLEFT", castbar, "TOPRIGHT", T.buttonspacing*2, 1)
					end
				end
			else
				castbar:SetHeight(height)
				if unit == "player" then
					castbar:Point("BOTTOM", TukuiBar1, "BOTTOM", 0, 180)
					castbar:SetWidth(width + T.buttonsize)
				elseif unit == "target" then
					castbar:Point("BOTTOM", TukuiBar1, "BOTTOM", 0, 250)
					castbar:SetWidth(width + T.buttonsize)
				end
			
				local castbarBg = CreateFrame("Frame", nil, castbar)
				castbarBg:Point("TOPLEFT", -2, 2)
				castbarBg:Point("BOTTOMRIGHT", 2, -2)
				castbarBg:SetFrameLevel(castbar:GetFrameLevel() - 1)
				castbarBg:SetTemplate("Default", true)
				castbarBg:CreateShadow("Default")			
					
				castbar.CustomTimeText = T.CustomCastTimeText
				castbar.CustomDelayText = T.CustomCastDelayText
				castbar.PostCastStart = T.CheckCast
				castbar.PostChannelStart = T.CheckChannel

				castbar.time = T.SetFontString(castbar, font, fonts, fontf)
				castbar.time:Point("RIGHT", castbar, "RIGHT", -T.buttonspacing, 0)
				castbar.time:SetTextColor(0.8, 0.75, 0.85)
				
				castbar.Text = T.SetFontString(castbar, font, fonts, fontf)
				castbar.Text:SetTextColor(0.8, 0.75, 0.85)
				
				if C["unitframes"].cbicons and not T.lowversion then
					castbar.button = CreateFrame("Frame", nil, castbar)
					castbar.button:SetTemplate("Default", true)
					castbar.button:CreateShadow("Default")

					castbar.icon = castbar.button:CreateTexture(nil, "ARTWORK")
					castbar.icon:Point("TOPLEFT", castbar.button, 2, -2)
					castbar.icon:Point("BOTTOMRIGHT", castbar.button, -2, 2)
					castbar.icon:SetTexCoord(0.08, 0.92, 0.08, .92)
					
					if unit == "player" then
						castbar.button:SetSize(height*2, height*2)
						castbar.button:SetPoint("BOTTOMLEFT", castbarBg, "TOPLEFT", 0,  T.buttonspacing)
					elseif unit == "target" then
						castbar.button:SetSize(height*2, height*2)
						castbar.button:SetPoint("TOPRIGHT", castbarBg, "BOTTOMRIGHT", 0, -T.buttonspacing)
					end
					
					castbar.Text:SetPoint("LEFT", castbar, "LEFT", T.buttonspacing, 0)
					castbar.Text:SetWidth(width - (T.buttonsize * 2))
				else
					castbar.Text:Point("CENTER", castbar)
					castbar.Text:SetWidth(width - T.buttonsize)
				end
			end
			
			-- cast bar latency on player
			if unit == "player" and C["unitframes"].cblatency == true then
				castbar.safezone = castbar:CreateTexture(nil, "ARTWORK")
				castbar.safezone:SetTexture(C["media"].blank)
				castbar.safezone:SetVertexColor(0.69, 0.31, 0.31, 0.75)
				castbar.SafeZone = castbar.safezone
			end
					
			self.Castbar = castbar
			self.Castbar.Time = castbar.time
			self.Castbar.Icon = castbar.icon
		end
		
		-- add combat feedback support
		if C["unitframes"].combatfeedback == true then
			local CombatFeedbackText 
			if C["theme"].pfont then
				CombatFeedbackText = T.SetFontString(health, font, fonts, fontf)
			else
				CombatFeedbackText = T.SetFontString(health, C["media"].dmgfont, fonts+2, fontf)
			end
			CombatFeedbackText:SetPoint("CENTER", 0, -2)
			CombatFeedbackText.colors = {
				DAMAGE = {0.69, 0.31, 0.31},
				CRUSHING = {0.69, 0.31, 0.31},
				CRITICAL = {0.69, 0.31, 0.31},
				GLANCING = {0.69, 0.31, 0.31},
				STANDARD = {0.84, 0.75, 0.65},
				IMMUNE = {0.84, 0.75, 0.65},
				ABSORB = {0.84, 0.75, 0.65},
				BLOCK = {0.84, 0.75, 0.65},
				RESIST = {0.84, 0.75, 0.65},
				MISS = {0.84, 0.75, 0.65},
				HEAL = {0.33, 0.59, 0.33},
				CRITHEAL = {0.33, 0.59, 0.33},
				ENERGIZE = {0.31, 0.45, 0.63},
				CRITENERGIZE = {0.31, 0.45, 0.63},
			}
			self.CombatFeedbackText = CombatFeedbackText
		end
		
		if (unit == "player") then
			-- vengeance bar
			if C["unitframes"].vengeance == true then
				local Vengeance = CreateFrame("StatusBar", self:GetName().."_Vengeance", self)
				Vengeance:SetFrameStrata("TOOLTIP")
				Vengeance:SetFrameLevel(20)
				Vengeance:Point("TOPLEFT", panel, 2, -2)
				Vengeance:Point("BOTTOMRIGHT", panel, -2, 2)
				Vengeance:SetStatusBarTexture(normTex)
				Vengeance:GetStatusBarTexture():SetHorizTile(false)
				Vengeance:SetStatusBarColor(unpack(C["media"].bgcolor))
				Vengeance:SetTemplate("Default", true)
				Vengeance:SetBackdropBorderColor(0, 0, 0, 0)
				Vengeance.Text = T.SetFontString(Vengeance, font, fonts, fontf)
				Vengeance.Text:Point("CENTER", self.panel, "CENTER", 0, 0)
				self.Vengeance = Vengeance
			end
			
			-- absorb tracker bar 		(future project)
			--[[if C["unitframes"].absorb == true then
				local Shields = CreateFrame("StatusBar", self:GetName().."_Shields", self)
				Shields:SetFrameStrata("TOOLTIP")
				Shields:SetFrameLevel(20)
				Shields:Point("TOPLEFT", panel, 2, -2)
				Shields:Point("BOTTOMRIGHT", panel, -2, 2)
				Shields:SetStatusBarTexture(normTex)
				Shields:GetStatusBarTexture():SetHorizTile(false)
				Shields:SetStatusBarColor(unpack(C["media"].bgcolor))
				Shields:SetTemplate("Default", true)
				Shields.Text = T.SetFontString(Shields, font, fonts, fontf)
				self.Shields = Shields
			end]]
				
			-- swing bar
			if C["unitframes"].swing == true then
				local Swing = CreateFrame("StatusBar", self:GetName().."_Swing", self)
				Swing:SetFrameLevel(2)
				if C["unitframes"].cbinside then
					Swing:Point("BOTTOMLEFT", TukuiInfoBottom, "TOPLEFT", 2, 3)
					Swing:Point("BOTTOMRIGHT", TukuiInfoBottom, "TOPRIGHT", -2, 3)
				else
					Swing:Point("TOPLEFT", self.Castbar, "BOTTOMLEFT", 0, -6)
					Swing:Point("TOPRIGHT", self.Castbar, "BOTTOMRIGHT", 0, -6)
				end
				Swing:Height(8)
				Swing:SetStatusBarTexture(normTex)
				Swing:GetStatusBarTexture():SetHorizTile(false)
				Swing:SetStatusBarColor(unpack(C["media"].bordercolor))	-- i like this color :)
						
				Swing.bg = CreateFrame("Frame", nil, Swing)
				Swing.bg:Point("TOPLEFT", -2, 2)
				Swing.bg:Point("BOTTOMRIGHT", 2, -2)
				Swing.bg:SetFrameLevel(Swing:GetFrameLevel() - 1)
				Swing.bg:SetTemplate("Default", true)
				Swing.bg:CreateShadow("Default")
				
				Swing.disableMelee = false
				Swing.disableRanged = false
				Swing.hideOoc = true
				
				self.Swing = Swing
			end
		end
		
		if C["unitframes"].healcomm then
			local mhpb = CreateFrame('StatusBar', nil, self.Health)
			mhpb:SetPoint('TOPLEFT', self.Health:GetStatusBarTexture(), 'TOPRIGHT', 0, 0)
			mhpb:SetPoint('BOTTOMLEFT', self.Health:GetStatusBarTexture(), 'BOTTOMRIGHT', 0, 0)
			if T.lowversion then
				mhpb:SetWidth(width/2)
			else
				mhpb:SetWidth(width)
			end
			mhpb:SetStatusBarTexture(normTex)
			mhpb:SetStatusBarColor(0, 1, 0.5, 0.25)
			mhpb:SetMinMaxValues(0,1)

			local ohpb = CreateFrame('StatusBar', nil, self.Health)
			ohpb:SetPoint('TOPLEFT', mhpb:GetStatusBarTexture(), 'TOPRIGHT', 0, 0)
			ohpb:SetPoint('BOTTOMLEFT', mhpb:GetStatusBarTexture(), 'BOTTOMRIGHT', 0, 0)
			ohpb:SetWidth(width)
			ohpb:SetStatusBarTexture(normTex)
			ohpb:SetStatusBarColor(0, 1, 0, 0.25)

			self.HealPrediction = {
				myBar = mhpb,
				otherBar = ohpb,
				maxOverflow = 1,
			}
		end
		
		-- player aggro
		if C["unitframes"].playeraggro == true then
			table.insert(self.__elements, T.UpdateThreat)
			self:RegisterEvent('PLAYER_TARGET_CHANGED', T.UpdateThreat)
			self:RegisterEvent('UNIT_THREAT_LIST_UPDATE', T.UpdateThreat)
			self:RegisterEvent('UNIT_THREAT_SITUATION_UPDATE', T.UpdateThreat)
		end
	end
	
	------------------------------------------------------------------------
	--	Target of Target, Focus and Pet unit layout
	------------------------------------------------------------------------
	
	if (unit == "targettarget") or (unit == "pet") or (unit == "focus") then
		-- health
		local health = CreateFrame("StatusBar", nil, self)
		health:SetHeight(height - 4)
		health:SetFrameLevel(5)
		health:SetPoint("TOPLEFT", T.mult, -T.mult)
		health:SetPoint("TOPRIGHT", -T.mult, T.mult)
		health:SetStatusBarTexture(normTex)
		self.Health = health
		
		local healthBg = health:CreateTexture(nil, "BORDER")
		healthBg:SetAllPoints()
		healthBg:SetTexture(unpack(C["media"].bgcolor))
		self.Health.bg = healthBg
				
		local healthB = CreateFrame("Frame", nil, health)
		healthB:SetFrameLevel(health:GetFrameLevel() - 1)
		healthB:Point("TOPLEFT", -2, 2)
		healthB:Point("BOTTOMRIGHT", 2, -2)
		healthB:SetTemplate("Default", true)
		healthB:CreateShadow("Default")
		self.Health.border = healthB
		
		-- power (only for focus)
		local power = CreateFrame("StatusBar", nil, self)
		self.Power = power
			
		if (unit == "focus") then
			health:SetHeight(T.buttonsize - 12)
		
			health.value = T.SetFontString(health, font, fonts, fontf)
			health.value:Point("RIGHT", health, "RIGHT", -6, 0)
			
			health.percent = T.SetFontString(health, font, fonts, fontf)			-- hack to disable error
			health.PostUpdate = T.PostUpdateHealth
			
			power.value = T.SetFontString(health, font, fonts, fontf)
			power.value:Point("LEFT", health, "LEFT", 6, 0)
			
			power.percent = T.SetFontString(health, font, fonts, fontf)			-- hack to disable error
			power.PreUpdate = T.PreUpdatePower
			power.PostUpdate = T.PostUpdatePower
		end
		
		health.frequentUpdates = true
		
		if C["unitframes"].showsmooth == true then
			health.Smooth = true
		end
		
		-- names
		local Name = health:CreateFontString(nil, "OVERLAY")
		Name:SetPoint("CENTER", health, "CENTER", 0, 0)
		Name:SetFont(font, fonts, fontf)
		Name:SetShadowOffset(1, -1)
		Name:SetJustifyH("CENTER")

		-- colors
		if C["unitframes"].unicolor == false then
			health.colorTapping = true
			health.colorDisconnected = true
			health.colorReaction = true
			health.colorClass = true
			healthBg.multiplier = 0.2
			
			self:Tag(Name, '[Tukui:namelong]')
		else
			health.colorTapping = false
			health.colorDisconnected = false
			health.colorClass = false
			health.colorReaction = false
			health:SetStatusBarColor(unpack(C["media"].barcolor))
			health.bg:SetVertexColor(unpack(C["media"].bgcolor))
			
			self:Tag(Name, '[Tukui:getnamecolor][Tukui:namelong]')
		end
		
		self.Name = Name
		
		-- auras
		if C["unitframes"].totdebuffs == true and unit == "targettarget" then
			local debuffs = CreateFrame("Frame", nil, health)
			debuffs:SetHeight(height)
			debuffs:SetWidth(width)
			debuffs.size = height
			debuffs.spacing = T.buttonspacing
			debuffs.num = 4
			debuffs:Point("LEFT", health, "RIGHT", 5, 0)
			debuffs.initialAnchor = "LEFT"
			debuffs["growth-x"] = "RIGHT"
			debuffs.PostCreateIcon = T.PostCreateAura
			debuffs.PostUpdateIcon = T.PostUpdateAura
			self.Debuffs = debuffs
		elseif C["unitframes"].petbuffs == true and unit == "pet" then
			local buffs = CreateFrame("Frame", nil, health)
			buffs:SetHeight(height)
			buffs:SetWidth(width)
			buffs.size = height
			buffs.spacing = T.buttonspacing
			buffs.num = 4
			buffs:Point("RIGHT", health, "LEFT", -5, 0)
			buffs.initialAnchor = "RIGHT"
			buffs["growth-x"] = "LEFT"
			buffs.PostCreateIcon = T.PostCreateAura
			buffs.PostUpdateIcon = T.PostUpdateAura
			buffs.onlyShowPlayer = true
			self.Buffs = buffs
		elseif C["unitframes"].focusdebuffs == true and unit == "focus" then
			local debuffs = CreateFrame("Frame", nil, health)
			debuffs:SetHeight(height * 1.8)
			debuffs:SetWidth(width * 1.5)
			debuffs.size = height * 1.8
			debuffs.spacing = T.buttonspacing
			debuffs.num = 8
			debuffs:Point("TOPLEFT", health, "BOTTOMLEFT", -2, -5)
			debuffs.initialAnchor = "BOTTOMLEFT"
			debuffs["growth-y"] = "DOWN"
			debuffs.PostCreateIcon = T.PostCreateAura
			debuffs.PostUpdateIcon = T.PostUpdateAura
			self.Debuffs = debuffs
		end
		
		-- focus castbar
		if C["unitframes"].unitcastbar and unit == "focus" then
			local castbar = CreateFrame("StatusBar", self:GetName().."_Castbar", self)
			castbar:Height(height)
			castbar:Width(width)
			castbar:SetStatusBarTexture(normTex)
			castbar:SetFrameLevel(6)
			castbar:SetPoint("CENTER", UIParent, "CENTER", 0, 200)		
			
			castbar.bg = CreateFrame("Frame", nil, castbar)
			castbar.bg:SetTemplate("Default", true)
			castbar.bg:CreateShadow("Default")			
			castbar.bg:Point("TOPLEFT", -2, 2)
			castbar.bg:Point("BOTTOMRIGHT", 2, -2)
			castbar.bg:SetFrameLevel(5)
			
			castbar.time = TukuiDB.SetFontString(castbar, font, fonts, fontf)
			castbar.time:Point("RIGHT", castbar, "RIGHT", -4, 0)
			castbar.time:SetTextColor(0.84, 0.75, 0.65)
			castbar.time:SetJustifyH("RIGHT")

			castbar.Text = TukuiDB.SetFontString(castbar, font, fonts, fontf)
			castbar.Text:Point("LEFT", castbar, "LEFT", 4, 0)
			castbar.Text:SetTextColor(0.84, 0.75, 0.65)
			
			castbar.CustomTimeText = CustomCastTimeText
			castbar.CustomDelayText = TukuiDB.CustomCastDelayText
			castbar.PostCastStart = TukuiDB.CheckCast
			castbar.PostChannelStart = TukuiDB.CheckChannel
			
			if C["unitframes"].cbicons then
				castbar.button = CreateFrame("Frame", nil, castbar)
				castbar.button:Height(height * 1.75)
				castbar.button:Width(height * 1.75)
				castbar.button:Point("CENTER", 0, height * 2)
				castbar.button:SetTemplate("Default", true)
				castbar.button:CreateShadow("Default")			

				castbar.icon = castbar.button:CreateTexture(nil, "ARTWORK")
				castbar.icon:Point("TOPLEFT", castbar.button, 2, -2)
				castbar.icon:Point("BOTTOMRIGHT", castbar.button, -2, 2)
				castbar.icon:SetTexCoord(0.08, 0.92, 0.08, .92)
			end

			self.Castbar = castbar
			self.Castbar.Time = castbar.time
			self.Castbar.Icon = castbar.icon
		end
		
		-- aggro
		if C["unitframes"].playeraggro and unit == "pet" then
			table.insert(self.__elements, T.UpdateThreat)
			self:RegisterEvent('PLAYER_TARGET_CHANGED', T.UpdateThreat)
			self:RegisterEvent('UNIT_THREAT_LIST_UPDATE', T.UpdateThreat)
			self:RegisterEvent('UNIT_THREAT_SITUATION_UPDATE', T.UpdateThreat)
		end

		-- update pet name, this should fix "UNKNOWN" pet names on pet unit.
		self:RegisterEvent("UNIT_PET", T.updateAllElements)
	end
	
	------------------------------------------------------------------------
	--	Focus target unit layout
	------------------------------------------------------------------------

	if (unit == "focustarget") then
		-- health 
		local health = CreateFrame('StatusBar', nil, self)
		health:SetHeight(height - 2)
		health:SetPoint("TOPLEFT", T.mult, -T.mult)
		health:SetPoint("TOPRIGHT", -T.mult, T.mult)
		health:SetStatusBarTexture(normTex)
		self.Health = health
		
		local healthBg = health:CreateTexture(nil, "BORDER")
		healthBg:SetAllPoints()
		healthBg:SetTexture(unpack(C["media"].bgcolor))
		self.Health.bg = healthBg
		
		local healthB = CreateFrame("Frame", nil, health)
		healthB:SetFrameLevel(health:GetFrameLevel() - 1)
		healthB:Point("TOPLEFT", -2, 2)
		healthB:Point("BOTTOMRIGHT", 2, -2)
		healthB:SetTemplate("Default", true)
		healthB:CreateShadow("Default")
		self.Health.border = healthB
				
		health.frequentUpdates = true
		
		if C["unitframes"].showsmooth == true then
			health.Smooth = true
		end
		
		-- name
		local Name = health:CreateFontString(nil, "OVERLAY")
		Name:Point("CENTER", health, "CENTER", 0, 0)
		Name:SetFont(font, fonts, fontf)
		Name:SetShadowOffset(1, -1)
		
		if C["unitframes"].unicolor == false then
			health.colorTapping = true
			health.colorDisconnected = true
			health.colorReaction = true
			health.colorClass = true
			healthBg.multiplier = 0.2
			
			self:Tag(Name, '[Tukui:nameshort]')
		else
			health.colorTapping = false
			health.colorDisconnected = false
			health.colorClass = false
			health.colorReaction = false
			health:SetStatusBarColor(unpack(C["media"].barcolor))
			health.bg:SetVertexColor(unpack(C["media"].bgcolor))
			
			self:Tag(Name, '[Tukui:getnamecolor][Tukui:nameshort]')
		end
				
		self.Name = Name
	end

	------------------------------------------------------------------------
	--	Arena or boss units layout (both mirror'd)
	------------------------------------------------------------------------
	
	if (unit and unit:find("arena%d") and C["arena"].unitframes == true) or (unit and unit:find("boss%d") and C["unitframes"].showboss == true) then
		-- Right-click focus on arena or boss units
		self:SetAttribute("type2", "focus")
		
		-- health 
		local health = CreateFrame('StatusBar', nil, self)
		health:SetHeight(height)
		health:SetPoint("TOPLEFT", T.mult, -T.mult)
		health:SetPoint("TOPRIGHT", -T.mult, T.mult)
		health:SetStatusBarTexture(normTex)
		self.Health = health
		
		local healthBg = health:CreateTexture(nil, "BORDER")
		healthBg:SetAllPoints()
		healthBg:SetTexture(unpack(C["media"].bgcolor))
		self.Health.bg = healthBg
		
		local healthB = CreateFrame("Frame", nil, health)
		healthB:SetFrameLevel(health:GetFrameLevel() - 1)
		healthB:Point("TOPLEFT", -2, 2)
		healthB:Point("BOTTOMRIGHT", 2, -2)
		healthB:SetTemplate("Default", true)
		healthB:CreateShadow("Default")
		self.Health.border = healthB
		
		-- power
		local power = CreateFrame('StatusBar', nil, self)
		power:SetHeight(height / 4)
		power:Point("TOPLEFT", healthB, "BOTTOMLEFT", 2, -4)
		power:Point("TOPRIGHT", healthB, "BOTTOMRIGHT", -2, -4)
		power:SetStatusBarTexture(C["media"].blank)
		self.Power = power
		
		local powerBg = power:CreateTexture(nil, "BORDER")
		powerBg:SetAllPoints()
		powerBg:SetTexture(unpack(C["media"].bgcolor))
		powerBg.multiplier = 0.3
		self.Power.bg = powerBg
		
		local powerB = CreateFrame("Frame", nil, power)
		powerB:SetFrameLevel(power:GetFrameLevel() - 1)
		powerB:Point("TOPLEFT", -2, 2)
		powerB:Point("BOTTOMRIGHT", 2, -2)
		powerB:SetTemplate("Default", true)
		powerB:CreateShadow("Default")
		self.Power.border = powerB	
		
		health.value = T.SetFontString(health, font, fonts, fontf)
		power.value = T.SetFontString(health, font, fonts, fontf)
		
		if not (unit and unit:find("boss%d")) then
			health.value:Point("LEFT", health, "LEFT", 6, 0)
			power.value:Point("RIGHT", health, "RIGHT", -6, 0)
		end
		
		health.percent = T.SetFontString(health, font, health:GetHeight(), fontf)
		health.percent:Point("RIGHT", health, "LEFT", -6, 0)
		health.percent:SetShadowOffset(-2,-2)
		health.PostUpdate = T.PostUpdateHealth
		
		power.percent = T.SetFontString(health, font, fonts, fontf)		-- hack to disable error
		power.PreUpdate = T.PreUpdatePower
		power.PostUpdate = T.PostUpdatePower
		
		health.frequentUpdates = true
		power.frequentUpdates = true
		
		if C["unitframes"].showsmooth == true then
			health.Smooth = true
			power.Smooth = true
		end
		
		-- names
		local Name = health:CreateFontString(nil, "OVERLAY")
		Name:Point("CENTER", health, "CENTER", 0, 0)
		Name:SetFont(font, fonts, fontf)
		Name:SetShadowOffset(1, -1)
		Name.frequentUpdates = 0.2		
				
		if C["unitframes"].unicolor == false then
			health.colorTapping = true
			health.colorDisconnected = true
			health.colorReaction = true
			health.colorClass = true
			healthBg.multiplier = 0.2
			
			power.colorPower = true
			
			self:Tag(Name, '[Tukui:namelong]')
		else
			health.colorTapping = false
			health.colorDisconnected = false
			health.colorClass = false
			health.colorReaction = false
			health:SetStatusBarColor(unpack(C["media"].barcolor))
			health.bg:SetVertexColor(unpack(C["media"].bgcolor))
			
			power.colorTapping = true
			power.colorDisconnected = true
			power.colorClass = true
			power.colorReaction = true
			
			self:Tag(Name, '[Tukui:getnamecolor][Tukui:namelong]')
		end
				
		self.Name = Name
		
		if (unit and unit:find("boss%d")) then
			-- alt power bar
			local AltPowerBar = CreateFrame("StatusBar", nil, power)
			AltPowerBar:SetFrameLevel(power:GetFrameLevel() + 4)
			AltPowerBar:SetStatusBarTexture(normTex)
			AltPowerBar:GetStatusBarTexture():SetHorizTile(false)
			AltPowerBar:SetStatusBarColor(1, 0, 0)

			AltPowerBar:Point("TOPLEFT", 2, -2)
			AltPowerBar:Point("TOPRIGHT", -2, 2)
			AltPowerBar:SetTemplate("Default", true)
			AltPowerBar:SetBackdropBorderColor(0, 0, 0, 0)

			self.AltPowerBar = AltPowerBar
			
			-- create buff of unit if they are boss units
			local buffs = CreateFrame("Frame", nil, health)
			buffs:SetHeight(asize)
			buffs:SetWidth(width)
			buffs.size = asize
			buffs.spacing = 2
			buffs.num = 10
			buffs:Point("BOTTOMLEFT", health, "TOPLEFT", -2, 5)
			buffs.initialAnchor = "TOPLEFT"
			buffs["growth-y"] = "UP"
			buffs.PostCreateIcon = T.PostCreateAura
			buffs.PostUpdateIcon = T.PostUpdateAura
			self.Buffs = buffs
			
			-- because it appear that sometime elements are not correct.
			self:HookScript("OnShow", T.updateAllElements)
		end

		-- trinket feature via trinket plugin
		if (C.arena.unitframes) and (unit and unit:find('arena%d')) then
			local Trinketbg = CreateFrame("Frame", nil, self)
			Trinketbg:SetHeight(height * 1.74)
			Trinketbg:SetWidth(height * 1.74)
			Trinketbg:Point("LEFT", self, "RIGHT", 6, -1.5)				
			Trinketbg:SetTemplate("Default", true)
			Trinketbg:CreateShadow("Default")
			Trinketbg:SetFrameLevel(0)
			self.Trinketbg = Trinketbg
			
			local Trinket = CreateFrame("Frame", nil, Trinketbg)
			Trinket:SetAllPoints(Trinketbg)
			Trinket:Point("TOPLEFT", Trinketbg, 2, -2)
			Trinket:Point("BOTTOMRIGHT", Trinketbg, -2, 2)
			Trinket:SetFrameLevel(1)
			Trinket.trinketUseAnnounce = true
			self.Trinket = Trinket
			
			-- create debuff for arena units
			local debuffs = CreateFrame("Frame", nil, health)
			debuffs:SetHeight(asize)
			debuffs:SetWidth(width)
			debuffs.size = asize
			debuffs.spacing = 2
			debuffs.num = 10
			debuffs:Point("BOTTOMLEFT", health, "TOPLEFT", -2, 5)
			debuffs.initialAnchor = "TOPLEFT"
			debuffs["growth-y"] = "UP"
			debuffs.PostCreateIcon = T.PostCreateAura
			debuffs.PostUpdateIcon = T.PostUpdateAura
			self.Debuffs = debuffs
		end
		
		-- boss & arena frames cast bar!
		local castbar = CreateFrame("StatusBar", self:GetName().."CastBar", power)
		castbar:SetPoint("TOPLEFT", power, "BOTTOMLEFT", 0, -T.buttonspacing * 2)
		castbar:SetPoint("TOPRIGHT", power, "BOTTOMRIGHT", 0, -T.buttonspacing * 2)
		
		castbar:SetHeight(height / 1.5)
		castbar:SetStatusBarTexture(normTex)
		castbar:SetFrameLevel(6)
		
		castbar.bg = CreateFrame("Frame", nil, castbar)
		castbar.bg:SetTemplate("Default", true)
		castbar.bg:CreateShadow("Default")
		--castbar.bg:SetBackdropColor(unpack(C["media"].backdropcolor))
		castbar.bg:Point("TOPLEFT", -2, 2)
		castbar.bg:Point("BOTTOMRIGHT", 2, -2)
		castbar.bg:SetFrameLevel(5)
		
		castbar.time = T.SetFontString(castbar, font, fonts, fontf)
		castbar.time:Point("RIGHT", castbar, "RIGHT", -4, 0)
		castbar.time:SetTextColor(0.84, 0.75, 0.65)
		castbar.time:SetJustifyH("RIGHT")
		castbar.CustomTimeText = T.CustomCastTimeText

		castbar.Text = T.SetFontString(castbar, font, fonts, fontf)
		castbar.Text:Point("LEFT", castbar, "LEFT", 4, 0)
		castbar.Text:SetTextColor(0.84, 0.75, 0.65)
		
		castbar.CustomDelayText = T.CustomCastDelayText
		castbar.PostCastStart = T.CheckCast
		castbar.PostChannelStart = T.CheckChannel
								
		castbar.button = CreateFrame("Frame", nil, castbar)
		castbar.button:Height(castbar:GetHeight() + 4)
		castbar.button:Width(castbar:GetHeight() + 4)
		castbar.button:Point("RIGHT", castbar, "LEFT",-5, 0)
		castbar.button:SetTemplate("Default", true)
		castbar.button:CreateShadow("Default")
		castbar.icon = castbar.button:CreateTexture(nil, "ARTWORK")
		castbar.icon:Point("TOPLEFT", castbar.button, 2, -2)
		castbar.icon:Point("BOTTOMRIGHT", castbar.button, -2, 2)
		castbar.icon:SetTexCoord(0.08, 0.92, 0.08, .92)

		self.Castbar = castbar
		self.Castbar.Time = castbar.time
		self.Castbar.Icon = castbar.icon
	end

	------------------------------------------------------------------------
	--	Main tanks and Main Assists layout (both mirror'd)
	------------------------------------------------------------------------
	
	if(self:GetParent():GetName():match"TukuiMainTank" or self:GetParent():GetName():match"TukuiMainAssist") then
		-- Right-click focus on maintank or mainassist units
		self:SetAttribute("type2", "focus")
		
		-- health 
		local health = CreateFrame('StatusBar', nil, self)
		health:SetHeight(height)
		health:SetPoint("TOPLEFT", T.mult, -T.mult)
		health:SetPoint("TOPRIGHT", -T.mult, T.mult)
		health:SetStatusBarTexture(normTex)
		self.Health = health
		
		local healthBg = health:CreateTexture(nil, "BORDER")
		healthBg:SetAllPoints()
		healthBg:SetTexture(unpack(C["media"].bgcolor))
		self.Health.bg = healthBg

		local healthB = CreateFrame("Frame", nil, health)
		healthB:SetFrameLevel(health:GetFrameLevel() - 1)
		healthB:Point("TOPLEFT", -2, 2)
		healthB:Point("BOTTOMRIGHT", 2, -2)
		healthB:SetTemplate("Default", true)
		healthB:CreateShadow("Default")
		self.Health.border = healthB
		
		health.frequentUpdates = true
		if C["unitframes"].showsmooth == true then
			health.Smooth = true
		end
		
		-- names
		local Name = health:CreateFontString(nil, "OVERLAY")
		Name:SetPoint("CENTER", health, "CENTER")
		Name:SetFont(font, fonts, fontf)
		Name:SetShadowOffset(1, -1)
		
		self.Name = Name
		
		if C["unitframes"].unicolor == false then
			health.colorTapping = true
			health.colorDisconnected = true
			health.colorReaction = true
			health.colorClass = true
			healthBg.multiplier = 0.2
			
			self:Tag(Name, '[Tukui:nameshort]')
		else
			health.colorTapping = false
			health.colorDisconnected = false
			health.colorClass = false
			health.colorReaction = false
			health:SetStatusBarColor(unpack(C["media"].barcolor))
			health.bg:SetVertexColor(unpack(C["media"].bgcolor))
			
			self:Tag(Name, '[Tukui:getnamecolor][Tukui:nameshort]')
		end
	end
	
	return self
end

------------------------------------------------------------------------
--	Default position of Tukui unitframes
------------------------------------------------------------------------

oUF:RegisterStyle('Tukui', Shared)

if C.actionbar.enable then
	-- player
	local player = oUF:Spawn('player', "TukuiPlayer")
	player:SetPoint("BOTTOMRIGHT", TukuiBar1, "TOPLEFT", T.Scale(32), height * 2.25)
	player:SetSize(width, height * 1.5) 

	-- target
	local target = oUF:Spawn('target', "TukuiTarget")
	target:SetPoint("BOTTOMLEFT", TukuiBar1, "TOPRIGHT", T.Scale(-26), height * 2.25)
	target:SetSize(width, height * 1.5)
else
	-- player
	local player = oUF:Spawn('player', "TukuiPlayer")
	player:Point("BOTTOM", UIParent, "BOTTOM", -250, 195)
	player:SetSize(width, height * 1.5) 

	-- target
	local target = oUF:Spawn('target', "TukuiTarget")
	target:Point("BOTTOM", UIParent, "BOTTOM", 250, 195)
	target:SetSize(width, height * 1.5)
end

-- focus
local focus = oUF:Spawn('focus', "TukuiFocus")
if C["others"].viewport then
	focus:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 5, -(height * 1.2))
else
	focus:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 5, -(height * 2))
end
focus:SetSize(width * 1.5, height)

-- tot
local tot = oUF:Spawn('targettarget', "TukuiTargetTarget")
tot:SetPoint("TOP", TukuiTarget, "BOTTOM", 0, -(height * 1.25))
tot:SetSize(width, height) 

-- pet
local pet = oUF:Spawn('pet', "TukuiPet")
pet:SetPoint("TOP", TukuiPlayer, "BOTTOM", 0, -(height * 1.25))
pet:SetSize(width, height) 

-- focus target
if C.unitframes.showfocustarget then
	local focustarget = oUF:Spawn("focustarget", "TukuiFocusTarget")
	focustarget:SetPoint("LEFT", TukuiFocus, "RIGHT", T.buttonspacing * 2, 0)
	focustarget:SetSize(width / 2, height)
end

-- arena
if C.arena.unitframes then
	local arena = {}
	for i = 1, 5 do
		arena[i] = oUF:Spawn("arena"..i, "TukuiArena"..i)
		if i == 1 then
			arena[i]:Point("BOTTOMLEFT", TukuiBar5, "TOPLEFT", 0, height*4.65)
		else
			arena[i]:Point("BOTTOM", arena[i-1], "TOP", 0, height*2.625)
		end
		arena[i]:SetSize(width, height * 1.5)
	end
end

-- boss
if C.unitframes.showboss then
	for i = 1,MAX_BOSS_FRAMES do
		local t_boss = _G["Boss"..i.."TargetFrame"]
		t_boss:UnregisterAllEvents()
		t_boss.Show = T.dummy
		t_boss:Hide()
		_G["Boss"..i.."TargetFrame".."HealthBar"]:UnregisterAllEvents()
		_G["Boss"..i.."TargetFrame".."ManaBar"]:UnregisterAllEvents()
	end

	local boss = {}
	for i = 1, MAX_BOSS_FRAMES do
		boss[i] = oUF:Spawn("boss"..i, "TukuiBoss"..i)
		if i == 1 then
			boss[i]:Point("BOTTOMLEFT", TukuiBar5, "TOPLEFT", 0, height*4.65)
		else
			boss[i]:Point("BOTTOM", boss[i-1], "TOP", 0, height*2.625)
		end
		boss[i]:SetSize(width, height * 1.5)
	end
end

-- main tank
if C.unitframes.maintank then
	local tank = oUF:SpawnHeader('TukuiMainTank', nil, 'raid',
		'oUF-initialConfigFunction', ([[
			self:SetWidth(%d)
			self:SetHeight(%d)
		]]):format(width / 2, height),
		'showRaid', true,
		'groupFilter', 'MAINTANK',
		'yOffset', 6,
		'point' , 'BOTTOM',
		'template', 'oUF_TukuiMtt'
	)
	tank:Point("BOTTOMLEFT", TukuiChatBackgroundLeft, "TOPLEFT", 0, 200)
end
 
-- main assist
if C.unitframes.mainassist then
	local assist = oUF:SpawnHeader("TukuiMainAssist", nil, 'raid',
		'oUF-initialConfigFunction', ([[
			self:SetWidth(%d)
			self:SetHeight(%d)
		]]):format(width / 2, height),
		'showRaid', true,
		'groupFilter', 'MAINASSIST',
		'yOffset', 6,
		'point' , 'BOTTOM',
		'template', 'oUF_TukuiMtt'
	)
	if C.unitframes.maintank == true then
		assist:SetPoint("TOPLEFT", TukuiMainTank, "BOTTOMLEFT", T.buttonspacing, -T.buttonspacing)
	else
		assist:Point("BOTTOMLEFT", TukuiChatBackgroundLeft, "TOPLEFT", 0, 200)
	end
end

-- Testui Command
local testui = TestUI or function() end
TestUI = function(msg)
	if msg == "arena" or msg == "a" then
		TukuiArena1:Show(); TukuiArena1.Hide = function() end; TukuiArena1.unit = "player"
		TukuiArena2:Show(); TukuiArena2.Hide = function() end; TukuiArena2.unit = "player"
		TukuiArena3:Show(); TukuiArena3.Hide = function() end; TukuiArena3.unit = "player"
		TukuiArena4:Show(); TukuiArena4.Hide = function() end; TukuiArena4.unit = "player"
		TukuiArena5:Show(); TukuiArena5.Hide = function() end; TukuiArena5.unit = "player"
	elseif msg == "boss" or msg == "b" then
		TukuiBoss1:Show(); TukuiBoss1.Hide = function() end; TukuiBoss1.unit = "player"
		TukuiBoss2:Show(); TukuiBoss2.Hide = function() end; TukuiBoss2.unit = "player"
		TukuiBoss3:Show(); TukuiBoss3.Hide = function() end; TukuiBoss3.unit = "player"
		TukuiBoss4:Show(); TukuiBoss4.Hide = function() end; TukuiBoss4.unit = "player"
	elseif msg == "pet" or msg == "p" then
		TukuiPet:Show(); TukuiPet.Hide = function() end; TukuiPet.unit = "player"
	elseif msg == "buffs" then -- better dont test it ^^
		UnitAura = function()
			-- name, rank, texture, count, dtype, duration, timeLeft, caster
			return 139, 'Rank 1', 'Interface\\Icons\\Spell_Holy_Penance', 1, 'Magic', 0, 0, "player"
		end
		if(oUF) then
			for i, v in pairs(oUF.units) do
				if(v.UNIT_AURA) then
					v:UNIT_AURA("UNIT_AURA", v.unit)
				end
			end
		end
	end
end
SlashCmdList.TestUI = TestUI
SLASH_TestUI1 = "/testui"

-- this is just a fake party to hide Blizzard frame if no Tukui raid layout are loaded.
local party = oUF:SpawnHeader("oUF_noParty", nil, "party", "showParty", true)

------------------------------------------------------------------------
-- Right-Click on unit frames menu. 
-- Doing this to remove SET_FOCUS eveywhere.
-- SET_FOCUS work only on Default unitframes.
-- Main Tank and Main Assist, use /maintank and /mainassist commands.
------------------------------------------------------------------------

do
	UnitPopupMenus["SELF"] = { "PVP_FLAG", "LOOT_METHOD", "LOOT_THRESHOLD", "OPT_OUT_LOOT_TITLE", "LOOT_PROMOTE", "DUNGEON_DIFFICULTY", "RAID_DIFFICULTY", "RESET_INSTANCES", "RAID_TARGET_ICON", "SELECT_ROLE", "CONVERT_TO_PARTY", "CONVERT_TO_RAID", "LEAVE", "CANCEL" };
	UnitPopupMenus["PET"] = { "PET_PAPERDOLL", "PET_RENAME", "PET_ABANDON", "PET_DISMISS", "CANCEL" };
	UnitPopupMenus["PARTY"] = { "MUTE", "UNMUTE", "PARTY_SILENCE", "PARTY_UNSILENCE", "RAID_SILENCE", "RAID_UNSILENCE", "BATTLEGROUND_SILENCE", "BATTLEGROUND_UNSILENCE", "WHISPER", "PROMOTE", "PROMOTE_GUIDE", "LOOT_PROMOTE", "VOTE_TO_KICK", "UNINVITE", "INSPECT", "ACHIEVEMENTS", "TRADE", "FOLLOW", "DUEL", "RAID_TARGET_ICON", "SELECT_ROLE", "PVP_REPORT_AFK", "RAF_SUMMON", "RAF_GRANT_LEVEL", "CANCEL" }
	UnitPopupMenus["PLAYER"] = { "WHISPER", "INSPECT", "INVITE", "ACHIEVEMENTS", "TRADE", "FOLLOW", "DUEL", "RAID_TARGET_ICON", "RAF_SUMMON", "RAF_GRANT_LEVEL", "CANCEL" }
	UnitPopupMenus["RAID_PLAYER"] = { "MUTE", "UNMUTE", "RAID_SILENCE", "RAID_UNSILENCE", "BATTLEGROUND_SILENCE", "BATTLEGROUND_UNSILENCE", "WHISPER", "INSPECT", "ACHIEVEMENTS", "TRADE", "FOLLOW", "DUEL", "RAID_TARGET_ICON", "SELECT_ROLE", "RAID_LEADER", "RAID_PROMOTE", "RAID_DEMOTE", "LOOT_PROMOTE", "RAID_REMOVE", "PVP_REPORT_AFK", "RAF_SUMMON", "RAF_GRANT_LEVEL", "CANCEL" };
	UnitPopupMenus["RAID"] = { "MUTE", "UNMUTE", "RAID_SILENCE", "RAID_UNSILENCE", "BATTLEGROUND_SILENCE", "BATTLEGROUND_UNSILENCE", "RAID_LEADER", "RAID_PROMOTE", "RAID_MAINTANK", "RAID_MAINASSIST", "RAID_TARGET_ICON", "LOOT_PROMOTE", "RAID_DEMOTE", "RAID_REMOVE", "PVP_REPORT_AFK", "CANCEL" };
	UnitPopupMenus["VEHICLE"] = { "RAID_TARGET_ICON", "VEHICLE_LEAVE", "CANCEL" }
	UnitPopupMenus["TARGET"] = { "RAID_TARGET_ICON", "CANCEL" }
	UnitPopupMenus["ARENAENEMY"] = { "CANCEL" }
	UnitPopupMenus["FOCUS"] = { "RAID_TARGET_ICON", "CANCEL" }
	UnitPopupMenus["BOSS"] = { "RAID_TARGET_ICON", "CANCEL" }
end