local ADDON_NAME, ns = ...
local oUF = oUFTukui or oUF
assert(oUF, "Tukui was unable to locate oUF install.")

ns._Objects = {}
ns._Headers = {}

local T, C, L = unpack(Tukui) -- Import: T - functions, constants, variables; C - config; L - locales
if not C["unitframes"].enable == true then return end

local font, fonts, fontf

if C["theme"].pfont then
	font, fonts, fontf = C["media"].pixelfont, C["theme"].pfonts, C["theme"].pfontf
else
	font, fonts, fontf = C["theme"].uff, C["theme"].uffs - 2, C["theme"].ufff
end

local normTex = C["media"].normTex

local height = T.buttonsize - 4
local width = (T.DataWidth / 5) - 5

local function Shared(self, unit)
	self.colors = T.oUF_colors
	self:RegisterForClicks("AnyUp")
	self:SetScript('OnEnter', UnitFrame_OnEnter)
	self:SetScript('OnLeave', UnitFrame_OnLeave)
	
	self.menu = T.SpawnMenu
	
	-- health
	local health = CreateFrame('StatusBar', nil, self)
	health:SetFrameLevel(5)
	health:Height(height)
	health:SetPoint("TOPLEFT", T.mult, -T.mult)
	health:SetPoint("TOPRIGHT", -T.mult, T.mult)
	health:SetStatusBarTexture(normTex)
	self.Health = health
		
	local healthBg = health:CreateTexture(nil, "BORDER")
	healthBg:SetAllPoints()
	healthBg:SetTexture(unpack(C["media"].bgcolor))
	self.Health.bg = healthBg
		
	local healthB = CreateFrame("Frame", nil, health)
	healthB:SetFrameLevel(health:GetFrameLevel() - 1)
	healthB:Point("TOPLEFT", -2, 2)
	healthB:Point("BOTTOMRIGHT", 2, -2)
	healthB:SetTemplate("Default", true)
	healthB:CreateShadow("Default")
	self.Health.border = healthB
	
	-- power
	local power = CreateFrame('StatusBar', nil, self)
	power:SetFrameLevel(self.Health:GetFrameLevel() + 2)
	power:SetHeight(height/5)
	power:Point("TOPLEFT", healthB, "BOTTOMLEFT", 8, 4)
	power:Point("TOPRIGHT", healthB, "BOTTOMRIGHT", -8, 4)
	power:SetStatusBarTexture(normTex)
	power:GetStatusBarTexture():SetHorizTile(false)
	self.Power = power
		
	local powerBg = power:CreateTexture(nil, "BORDER")
	powerBg:SetAllPoints()
	powerBg:SetTexture(unpack(C["media"].bgcolor))
	powerBg.multiplier = 0.3
	self.Power.bg = powerBg
		
	local powerB = CreateFrame("Frame", nil, power)
	powerB:SetFrameLevel(power:GetFrameLevel() - 1)
	powerB:Point("TOPLEFT", -2, 2)
	powerB:Point("BOTTOMRIGHT", 2, -2)
	powerB:SetTemplate("Default", true)
	powerB:CreateShadow("Default")
	self.Power.border = powerB	
	
	health.frequentUpdates = true
	power.frequentUpdates = true
		
	if C["unitframes"].unicolor == false then
		health.colorTapping = true
		health.colorDisconnected = true
		health.colorReaction = true
		health.colorClass = true
		healthBg.multiplier = 0.3
			
		power.colorPower = true
	else
		health.colorTapping = false
		health.colorDisconnected = false
		health.colorClass = false
		health.colorReaction = false
		health:SetStatusBarColor(unpack(C["media"].barcolor))
		health.bg:SetVertexColor(unpack(C["media"].bgcolor))
			
		power.multiplier = 0.5
		power.colorTapping = true
		power.colorDisconnected = true
		power.colorClass = true
		power.colorReaction = true
	end
	
	local name = health:CreateFontString(nil, "OVERLAY")
    name:SetPoint("CENTER", health, "CENTER", 0, T.buttonspacing)
	name:SetFont(font, fonts, fontf)
	name:SetShadowOffset(1, -1)
	if C["unitframes"].unicolor == false then
		self:Tag(name, '[Tukui:namemedium]')
	else
		self:Tag(name, '[Tukui:getnamecolor][Tukui:namemedium]')
	end
	self.Name = name
	
	if C["unitframes"].showsymbols == true then
		RaidIcon = health:CreateTexture(nil, 'OVERLAY')
		RaidIcon:Height(14)
		RaidIcon:Width(14)
		RaidIcon:SetPoint("TOPRIGHT", self)
		RaidIcon:SetTexture("Interface\\AddOns\\Tukui\\medias\\textures\\raidicons.blp") -- thx hankthetank for texture
		self.RaidIcon = RaidIcon
	end
		
	local LFDRole = health:CreateTexture(nil, "OVERLAY")
	LFDRole:Height(6)
	LFDRole:Width(6)
	LFDRole:Point("TOPLEFT", 2, -2)
	LFDRole:SetTexture("Interface\\AddOns\\Tukui\\medias\\textures\\lfdicons.blp")
	self.LFDRole = LFDRole
	
	local ReadyCheck = health:CreateTexture(nil, "OVERLAY")
	ReadyCheck:Height(12)
	ReadyCheck:Width(12)
	ReadyCheck:SetPoint('CENTER')
	self.ReadyCheck = ReadyCheck
	
	--local picon = self.Health:CreateTexture(nil, 'OVERLAY')
	--picon:SetPoint('CENTER', self.Health)
	--picon:SetSize(16, 16)
	--picon:SetTexture[[Interface\AddOns\Tukui\medias\textures\picon]]
	--picon.Override = T.Phasing
	--self.PhaseIcon = picon
	
	local dbh = self.Health:CreateTexture(nil, "OVERLAY", healthBg)
	dbh:SetAllPoints(self)
	dbh:SetTexture(normTex)
	dbh:SetBlendMode("ADD")
	dbh:SetVertexColor(0,0,0,0)
	self.DebuffHighlight = dbh
	self.DebuffHighlightFilter = true
	self.DebuffHighlightAlpha = 0.75

	if C["unitframes"].showrange == true then
		local range = {insideAlpha = 1, outsideAlpha = C["unitframes"].raidalphaoor}
		self.Range = range
	end
	
	if C["unitframes"].aggro == true then
		table.insert(self.__elements, T.UpdateThreat)
		self:RegisterEvent('PLAYER_TARGET_CHANGED', T.UpdateThreat)
		self:RegisterEvent('UNIT_THREAT_LIST_UPDATE', T.UpdateThreat)
		self:RegisterEvent('UNIT_THREAT_SITUATION_UPDATE', T.UpdateThreat)
    end
	
	if C["unitframes"].raidunitdebuffwatch == true then
		-- Raid Debuffs (big middle icon)
		local RaidDebuffs = CreateFrame('Frame', nil, self)
		RaidDebuffs:Height((height-4)*C["unitframes"].gridscale)
		RaidDebuffs:Width((height-4)*C["unitframes"].gridscale)
		RaidDebuffs:Point('CENTER', health, 1,0)
		RaidDebuffs:SetFrameStrata(health:GetFrameStrata())
		RaidDebuffs:SetFrameLevel(health:GetFrameLevel() + 2)
		
		RaidDebuffs:SetTemplate("Default")
		
		RaidDebuffs.icon = RaidDebuffs:CreateTexture(nil, 'OVERLAY')
		RaidDebuffs.icon:SetTexCoord(.1,.9,.1,.9)
		RaidDebuffs.icon:Point("TOPLEFT", 2, -2)
		RaidDebuffs.icon:Point("BOTTOMRIGHT", -2, 2)

		RaidDebuffs.count = RaidDebuffs:CreateFontString(nil, 'OVERLAY')
		RaidDebuffs.count:SetFont(font, fonts-4, fontf)
		RaidDebuffs.count:SetPoint('BOTTOMRIGHT', RaidDebuffs, 'BOTTOMRIGHT', 0, 2)
		RaidDebuffs.count:SetTextColor(1, .9, 0)
		
		self.RaidDebuffs = RaidDebuffs
    end

    -- execute an update on every raids unit if party or raid member changed
    -- should fix issues with names/symbols/etc not updating introduced with 4.0.3 patch
    self:RegisterEvent("PARTY_MEMBERS_CHANGED", T.updateAllElements)
    self:RegisterEvent("RAID_ROSTER_UPDATE", T.updateAllElements)
	
	return self
end

oUF:RegisterStyle('TukuiDps', Shared)
oUF:Factory(function(self)
	oUF:SetActiveStyle("TukuiDps")

	local group = self:SpawnHeader("TukuiDps", nil, "solo,party,raid", 
	'oUF-initialConfigFunction', [[
		local header = self:GetParent()
		self:SetWidth(header:GetAttribute('initial-width'))
		self:SetHeight(header:GetAttribute('initial-height'))
	]],
	"initial-width", width,
	"initial-height", height,
	"showParty", true, 
	"showPlayer", C["unitframes"].showplayerinparty, 
	"showRaid", true, 
	"groupFilter", "1,2,3,4,5,6,7,8", 
	"groupingOrder", "1,2,3,4,5,6,7,8", 
	"groupBy", "GROUP", 
	"xOffset", T.Scale(6),
	"point", "LEFT",
	"maxColumns", 8,
	"unitsPerColumn", 5,
	"columnSpacing", T.Scale(10),
	"columnAnchorPoint", "BOTTOM")
	group:Point("BOTTOMLEFT", TukuiChatBackgroundLeft, "TOPLEFT", 1, 8)
end)

-- only show 5 groups in raid (25 man raid)
local MaxGroup = CreateFrame("Frame")
MaxGroup:RegisterEvent("PLAYER_ENTERING_WORLD")
MaxGroup:RegisterEvent("ZONE_CHANGED_NEW_AREA")
MaxGroup:SetScript("OnEvent", function(self)
	local inInstance, instanceType = IsInInstance()
	local _, _, _, _, maxPlayers, _, _ = GetInstanceInfo()
	if inInstance and instanceType == "raid" and maxPlayers ~= 40 then
		TukuiDps:SetAttribute("groupFilter", "1,2,3,4,5")
	else
		TukuiDps:SetAttribute("groupFilter", "1,2,3,4,5,6,7,8")
	end
end)