local ADDON_NAME, ns = ...
local oUF = oUFTukui or oUF
assert(oUF, "Tukui was unable to locate oUF install.")

ns._Objects = {}
ns._Headers = {}

local T, C, L = unpack(Tukui) -- Import: T - functions, constants, variables; C - config; L - locales
if not C["unitframes"].enable == true then return end

local font, fonts, fontf

if C["theme"].pfont then
	font, fonts, fontf = C["media"].pixelfont, C["theme"].pfonts, C["theme"].pfontf
else
	font, fonts, fontf = C["theme"].uff, C["theme"].uffs - 2, C["theme"].ufff
end

local normTex = C["media"].normTex

local scale = C["unitframes"].gridscale
local height = T.buttonsize * scale
local width = T.DataWidth * scale

local function Shared(self, unit)
	self.colors = T.colors
	self:RegisterForClicks("AnyUp")
	self:SetScript('OnEnter', UnitFrame_OnEnter)
	self:SetScript('OnLeave', UnitFrame_OnLeave)
	
	self.menu = T.SpawnMenu
	
	-- health
	local health = CreateFrame('StatusBar', nil, self)
	if unit:find("partypet") then
		health:SetHeight((height / 2)*scale)
	else
		health:SetHeight((height + 10)*scale)
	end
	health:SetPoint("TOPLEFT", T.mult, -T.mult)
	health:SetPoint("TOPRIGHT", -T.mult, T.mult)
	health:SetStatusBarTexture(normTex)
	self.Health = health
		
	local healthBg = health:CreateTexture(nil, "BORDER")
	healthBg:SetAllPoints()
	healthBg:SetTexture(unpack(C["media"].bgcolor))
	self.Health.bg = healthBg
		
	local healthB = CreateFrame("Frame", nil, health)
	healthB:SetFrameLevel(health:GetFrameLevel() - 1)
	healthB:Point("TOPLEFT", -2, 2)
	healthB:Point("BOTTOMRIGHT", 2, -2)
	healthB:SetTemplate("Default", true)
	healthB:CreateShadow("Default")
	self.Health.border = healthB
		
	-- power
	local power = CreateFrame('StatusBar', nil, self)
	if unit:find("partypet") then
		power:SetHeight(0)
	else
		power:SetHeight((height/4.5)*scale)
	end
	power:SetFrameLevel(self.Health:GetFrameLevel() + 2)
	power:Point("TOPLEFT", healthB, "BOTTOMLEFT", 8, 4)
	power:Point("TOPRIGHT", healthB, "BOTTOMRIGHT", -8, 4)
	power:SetStatusBarTexture(normTex)
	power:GetStatusBarTexture():SetHorizTile(false)
	self.Power = power
		
	local powerBg = power:CreateTexture(nil, "BORDER")
	powerBg:SetAllPoints()
	powerBg:SetTexture(unpack(C["media"].bgcolor))
	powerBg.multiplier = 0.3
	self.Power.bg = powerBg
	
	local powerB = CreateFrame("Frame", nil, power)
	powerB:SetFrameLevel(power:GetFrameLevel() - 1)
	powerB:Point("TOPLEFT", -2, 2)
	powerB:Point("BOTTOMRIGHT", 2, -2)
	powerB:SetTemplate("Default", true)
	powerB:CreateShadow("Default")
	self.Power.border = powerB	
	
	if not unit:find("partypet") then
		health.value = health:CreateFontString(nil, "OVERLAY")
		health.value:SetPoint("CENTER", health, "CENTER", 0, -3)
		health.value:SetFont(font, (fonts + 2)*scale, fontf)
		health.value:SetTextColor(1, 1, 1)
		health.value:SetShadowOffset(1, -1)
		self.Health.value = health.value
		health.PostUpdate = T.PostUpdateHealthRaid
	end
		
	health.frequentUpdates = true
	power.frequentUpdates = true
	
	if C["unitframes"].unicolor == false then
		health.colorTapping = true
		health.colorDisconnected = true
		health.colorReaction = true
		health.colorClass = true
		healthBg.multiplier = 0.3
			
		power.colorPower = true
	else
		health.colorTapping = false
		health.colorDisconnected = false
		health.colorClass = false
		health.colorReaction = false
		health:SetStatusBarColor(unpack(C["media"].barcolor))
		health.bg:SetVertexColor(unpack(C["media"].bgcolor))
			
		power.multiplier = 0.5
		power.colorTapping = true
		power.colorDisconnected = true
		power.colorClass = true
		power.colorReaction = true
	end
	
	local name = health:CreateFontString(nil, "OVERLAY")
	if unit:find("partypet") then
		name:Point("CENTER", health, "CENTER", 0, 0)
	else
		name:Point("TOP", health, "TOP", 0, -2)
	end
	name:SetFont(font, fonts*scale, fontf)
	name:SetShadowOffset(1, -1)
	if C["unitframes"].unicolor == false then
		self:Tag(name, '[Tukui:namemedium]')
	else
		self:Tag(name, '[Tukui:getnamecolor][Tukui:namemedium]')
	end
	self.Name = name
	
    local leader = health:CreateTexture(nil, "OVERLAY")
    leader:Height(12*scale)
    leader:Width(12*scale)
    leader:SetPoint("TOPLEFT", 0, 6)
	self.Leader = leader
	
    local LFDRole = health:CreateTexture(nil, "OVERLAY")
    LFDRole:Height(6*scale)
    LFDRole:Width(6*scale)
	LFDRole:Point("TOPRIGHT", -2, -2)
	LFDRole:SetTexture("Interface\\AddOns\\Tukui\\medias\\textures\\lfdicons.blp")
	self.LFDRole = LFDRole
	
    local MasterLooter = health:CreateTexture(nil, "OVERLAY")
    MasterLooter:Height(12*scale)
    MasterLooter:Width(12*scale)
	self.MasterLooter = MasterLooter
    self:RegisterEvent("PARTY_LEADER_CHANGED", T.MLAnchorUpdate)
    self:RegisterEvent("PARTY_MEMBERS_CHANGED", T.MLAnchorUpdate)
		
	if C["unitframes"].showsymbols == true then
		local RaidIcon = health:CreateTexture(nil, 'OVERLAY')
		RaidIcon:Height(18*scale)
		RaidIcon:Width(18*scale)
		RaidIcon:SetPoint("TOPRIGHT", self)
		RaidIcon:SetTexture("Interface\\AddOns\\Tukui\\medias\\textures\\raidicons.blp") -- thx hankthetank for texture
		self.RaidIcon = RaidIcon
	end
	
	local ReadyCheck = self.Power:CreateTexture(nil, "OVERLAY")
	ReadyCheck:Height(12*scale)
	ReadyCheck:Width(12*scale)
	ReadyCheck:SetPoint('CENTER')
	self.ReadyCheck = ReadyCheck
	
	--local picon = self.Health:CreateTexture(nil, 'OVERLAY')
	--picon:SetPoint('CENTER', self.Health)
	--picon:SetSize(16, 16)
	--picon:SetTexture[[Interface\AddOns\Tukui\medias\textures\picon]]
	--picon.Override = T.Phasing
	--self.PhaseIcon = picon
	
	if not unit:find("partypet") then
		local dbh = self.Health:CreateTexture(nil, "OVERLAY", healthBg)
		dbh:SetAllPoints(self)
		dbh:SetTexture(normTex)
		dbh:SetBlendMode("ADD")
		dbh:SetVertexColor(0,0,0,0)
		self.DebuffHighlight = dbh
		self.DebuffHighlightFilter = true
		self.DebuffHighlightAlpha = 0.75
	end
	
	if C["unitframes"].showrange == true then
		local range = {insideAlpha = 1, outsideAlpha = C["unitframes"].raidalphaoor}
		self.Range = range
	end
		
	if C["unitframes"].aggro == true then
		table.insert(self.__elements, T.UpdateThreat)
		self:RegisterEvent('PLAYER_TARGET_CHANGED', T.UpdateThreat)
		self:RegisterEvent('UNIT_THREAT_LIST_UPDATE', T.UpdateThreat)
		self:RegisterEvent('UNIT_THREAT_SITUATION_UPDATE', T.UpdateThreat)
    end
	
	if C["unitframes"].healcomm and not unit:find("partypet") then
		local mhpb = CreateFrame('StatusBar', nil, self.Health)
		mhpb:SetPoint('TOPLEFT', self.Health:GetStatusBarTexture(), 'TOPRIGHT', 0, 0)
		mhpb:SetPoint('BOTTOMLEFT', self.Health:GetStatusBarTexture(), 'BOTTOMRIGHT', 0, 4)
		mhpb:SetWidth((width / 5)*scale)
		mhpb:SetStatusBarTexture(normTex)
		mhpb:SetStatusBarColor(0, 1, 0.5, 0.5)

		local ohpb = CreateFrame('StatusBar', nil, self.Health)
		ohpb:SetPoint('TOPLEFT', mhpb:GetStatusBarTexture(), 'TOPRIGHT', 0, 0)
		ohpb:SetPoint('BOTTOMLEFT', mhpb:GetStatusBarTexture(), 'BOTTOMRIGHT', 0, 0)
		ohpb:SetWidth((width / 5)*scale)
		ohpb:SetStatusBarTexture(normTex)
		ohpb:SetStatusBarColor(0, 1, 0, 0.5)

		self.HealPrediction = {
			myBar = mhpb,
			otherBar = ohpb,
			maxOverflow = 1,
		}
	end
	
	if C["unitframes"].raidunitdebuffwatch == true and not unit:find("partypet") then
		-- AuraWatch (corner icon)
		T.createAuraWatch(self,unit)
		
		-- Raid Debuffs (big middle icon)
		local RaidDebuffs = CreateFrame('Frame', nil, self)
		RaidDebuffs:Height(height - 2)
		RaidDebuffs:Width(height - 2)
		RaidDebuffs:Point('TOP', health, 0, -1)
		RaidDebuffs:SetFrameStrata(health:GetFrameStrata())
		RaidDebuffs:SetFrameLevel(health:GetFrameLevel() + 2)
		
		RaidDebuffs:SetTemplate("Default")
		
		RaidDebuffs.icon = RaidDebuffs:CreateTexture(nil, 'OVERLAY')
		RaidDebuffs.icon:SetTexCoord(.1,.9,.1,.9)
		RaidDebuffs.icon:Point("TOPLEFT", 2, -2)
		RaidDebuffs.icon:Point("BOTTOMRIGHT", -2, 2)
		
		if C["unitframes"].raidtimer then
			RaidDebuffs.cd = CreateFrame('Cooldown', nil, RaidDebuffs)
			RaidDebuffs.cd:SetPoint("TOPLEFT", T.Scale(2), T.Scale(-2))
			RaidDebuffs.cd:SetPoint("BOTTOMRIGHT", T.Scale(-2), T.Scale(2))
		end

		RaidDebuffs.count = RaidDebuffs:CreateFontString(nil, 'OVERLAY')
		RaidDebuffs.count:SetFont(font, fonts-4, fontf)
		RaidDebuffs.count:SetPoint('BOTTOMRIGHT', RaidDebuffs, 'BOTTOMRIGHT', 0, 2)
		RaidDebuffs.count:SetTextColor(1, .9, 0)
		
		self.RaidDebuffs = RaidDebuffs
    end

    -- execute an update on every raids unit if party or raid member changed
    -- should fix issues with names/symbols/etc not updating introduced with 4.0.3 patch
    self:RegisterEvent("PARTY_MEMBERS_CHANGED", T.updateAllElements)
    self:RegisterEvent("RAID_ROSTER_UPDATE", T.updateAllElements)
	
	return self
end

oUF:RegisterStyle('TukuiHeal', Shared)
oUF:Factory(function(self)
	oUF:SetActiveStyle("TukuiHeal")

	local group = self:SpawnHeader("TukuiHeal", nil, "solo,party,raid", 
	'oUF-initialConfigFunction', [[
		local header = self:GetParent()
		self:SetWidth(header:GetAttribute('initial-width'))
		self:SetHeight(header:GetAttribute('initial-height'))
	]],
	"initial-width", (width / 5)*scale,
	"initial-height", (height + 10)*scale,
	"showParty", true, 
	"showPlayer", C["unitframes"].showplayerinparty, 
	"showRaid", true, 
	"groupFilter", "1,2,3,4,5,6,7,8", 
	"groupingOrder", "1,2,3,4,5,6,7,8", 
	"groupBy", "GROUP", 
	"xOffset", T.Scale(-6),
	"point", "RIGHT",
	"maxColumns", 8,
	"unitsPerColumn", 5,
	"columnSpacing", T.Scale(12),
	"columnAnchorPoint", "TOP")
	group:Point("BOTTOMRIGHT", TukuiPlayer, "TOPRIGHT", 0, 230)
	
	local pets = {}
		pets[1] = oUF:Spawn('partypet1', 'TukuiPartyPet1') 
		pets[1]:SetSize(group:GetAttribute('initial-width'), height / 2)
		if C["unitframes"].showplayerinparty then
			pets[1]:SetPoint('BOTTOMRIGHT', group, 'TOPRIGHT', -(pets[1]:GetWidth() + (T.Scale(6))), T.Scale(8))
		else
			pets[1]:SetPoint('BOTTOMRIGHT', group, 'TOPRIGHT', 0, T.Scale(8))
		end
	for i = 2, 4 do 
		pets[i] = oUF:Spawn('partypet'..i, 'TukuiPartyPet'..i) 
		pets[i]:SetSize(group:GetAttribute('initial-width'), height / 2)
		pets[i]:SetPoint('TOPRIGHT', pets[i-1], 'TOPLEFT', T.Scale(-6), 0)
	end
end)

-- Only enable petframes for party frames
local RaidMove = CreateFrame("Frame")
RaidMove:RegisterEvent("PLAYER_ENTERING_WORLD")
RaidMove:RegisterEvent("RAID_ROSTER_UPDATE")
RaidMove:RegisterEvent("PARTY_LEADER_CHANGED")
RaidMove:RegisterEvent("PARTY_MEMBERS_CHANGED")
RaidMove:SetScript("OnEvent", function(self)
	if InCombatLockdown() then
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
	else
		self:UnregisterEvent("PLAYER_REGEN_ENABLED")
		local numraid = GetNumRaidMembers()
		local numparty = GetNumPartyMembers()
		if numparty > 0 and numraid == 0 or numraid > 0 and numraid <= 5 then
			for i,v in ipairs(pets) do v:Enable() end
		elseif numraid > 5 and numraid <= 25 then
			for i,v in ipairs(pets) do v:Disable() end
		elseif numraid > 25 then
			scale = { }
			scale = 0.75
			for i,v in ipairs(pets) do v:Disable() end
		end
	end
end)

-- only show 5 groups in raid (25 man raid)
local MaxGroup = CreateFrame("Frame")
MaxGroup:RegisterEvent("PLAYER_ENTERING_WORLD")
MaxGroup:RegisterEvent("ZONE_CHANGED_NEW_AREA")
MaxGroup:SetScript("OnEvent", function(self)
	local inInstance, instanceType = IsInInstance()
	local _, _, _, _, maxPlayers, _, _ = GetInstanceInfo()
	if inInstance and instanceType == "raid" and maxPlayers ~= 40 then
		TukuiHeal:SetAttribute("groupFilter", "1,2,3,4,5")
	else
		TukuiHeal:SetAttribute("groupFilter", "1,2,3,4,5,6,7,8")
	end
end)